#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#include <gtksourceview/gtksourcetag.h>

#include "if_main.h"
#include "if_preferences.h"
#include "support.h"
#include "main.h"
#include "editor.h"
#include "debug.h"
#include "g_options.h"

void on_chkPrefEmbedEditor_clicked(GtkButton * button, gpointer user_data);
void on_chkPrefAltEditor_clicked(GtkButton * button, gpointer user_data);

void on_chkTransparentBg_clicked(GtkButton * button, gpointer user_data);

void on_chkForeground_clicked(GtkButton * button, gpointer user_data);
void on_chkBackground_clicked(GtkButton * button, gpointer user_data);
void on_cmdStyle_clicked(GtkButton * button, gpointer user_data);

void on_editor_color_row_changed(GtkTreeSelection *selection, gpointer data);
void on_clbForeground_color_set(GtkColorButton *widget, gpointer data);
void on_clbBackground_color_set(GtkColorButton *widget, gpointer data);

void preferences_load_colors();

static GladeXML *xmlPreferences;
static options *prefs;

static const keyvaluepair color_mapping[] = {
  {"clbFgDefault", "fg_default"},
  {"clbFgDefaultH", "fg_default_h"},
  {"clbBgDefault", "bg_default"},
  
  {"clbFgBlack", "fg_black"},
  {"clbFgBlackH", "fg_black_h"},
  {"clbFgRed", "fg_red"},
  {"clbFgRedH", "fg_red_h"},
  {"clbFgGreen", "fg_green"},
  {"clbFgGreenH", "fg_green_h"},
  {"clbFgYellow", "fg_yellow"},
  {"clbFgYellowH", "fg_yellow_h"},
  {"clbFgBlue", "fg_blue"},
  {"clbFgBlueH", "fg_blue_h"},
  {"clbFgPurple", "fg_purple"},
  {"clbFgPurpleH", "fg_purple_h"},
  {"clbFgCyan", "fg_cyan"},
  {"clbFgCyanH", "fg_cyan_h"},
  {"clbFgWhite", "fg_white"},
  {"clbFgWhiteH", "fg_white_h"},
  
  {"clbBgBlack", "bg_black"},
  {"clbBgRed", "bg_red"},
  {"clbBgGreen", "bg_green"},
  {"clbBgYellow", "bg_yellow"},
  {"clbBgBlue", "bg_blue"},
  {"clbBgPurple", "bg_purple"},
  {"clbBgCyan", "bg_cyan"},
  {"clbBgWhite", "bg_white"}
};

enum {
  EDITOR_NAME,
  EDITOR_STYLE,
  EDITOR_ID,
  EDITOR_N_COLUMNS
};

GtkWidget *
if_preferences_get_widget(gchar * name) {
  return glade_xml_get_widget(xmlPreferences, name);
}

void
if_preferences_editor_list_init() {
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkTreeView *tv = GTK_TREE_VIEW(if_preferences_get_widget("tvEditorColors"));
  GtkListStore *store = gtk_list_store_new(EDITOR_N_COLUMNS, G_TYPE_STRING, 
                                           GTK_TYPE_SOURCE_TAG_STYLE, 
                                           G_TYPE_STRING);
  GtkTreeModel *model =
    gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
  GtkTreeSelection *select;
  
  gtk_tree_view_set_model(tv, model);
  select = gtk_tree_view_get_selection(tv);
  
  g_signal_connect((gpointer)select, "changed", 
                   G_CALLBACK(on_editor_color_row_changed), NULL);
  
  renderer = gtk_cell_renderer_text_new();
  column =
    gtk_tree_view_column_new_with_attributes(_("Tag"), renderer, "text",
                                             EDITOR_NAME, NULL);
  gtk_tree_view_append_column(tv, column);
  
  renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes(_("Style"), renderer, NULL);
  gtk_tree_view_column_set_visible(column, FALSE);
  
  gtk_tree_view_append_column(tv, column);  

  renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes(_("Id"), renderer, NULL);
  gtk_tree_view_column_set_visible(column, FALSE);
  
  gtk_tree_view_append_column(tv, column);
    
  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), EDITOR_NAME,
                                       GTK_SORT_ASCENDING);
}

/** \brief initialise editor colors
  *
  * initialises the editor colors list view
  */
void
if_preferences_editor_colors_init() {
  GtkSourceTag *st;
  GtkSourceTagStyle *sty;
  
  GSList *tags, *tag;
  gchar *name, *id;
  GtkTreeIter iter;
  GtkListStore *store;
  GtkSourceLanguage *lang;
  
  lang = editor_get_language();

  if (lang) {
    // Create column ed
    if_preferences_editor_list_init();
    store = GTK_LIST_STORE(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(
                             gtk_tree_view_get_model(GTK_TREE_VIEW(
                               if_preferences_get_widget("tvEditorColors"))))));
    tags = gtk_source_language_get_tags(lang);
    
    for (tag = tags; tag; tag = tag->next) {
      st = GTK_SOURCE_TAG(tag->data);
      sty = gtk_source_tag_get_style(st);
      
      g_object_get(G_OBJECT(st), "name", &name, "id", &id, NULL);
      
      gtk_list_store_prepend(store, &iter);
      gtk_list_store_set(store, &iter, EDITOR_NAME, name, 
                         EDITOR_STYLE, sty, EDITOR_ID, id, -1);

      g_free(name);
      g_free(id);
    }
    
    g_slist_free(tags);
  }
}

GtkWidget *
create_dlgPreferences(options * opt) {
  gchar *altEditor;

  xmlPreferences = glade_xml_new(G_XML, "dlgPreferences", NULL);
  prefs = opt;

  if_preferences_editor_colors_init();

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                               (if_preferences_get_widget("chkBold")),
                               options_get_int(opt, "bold-colors"));
                               
  preferences_load_colors();

  gtk_font_button_set_font_name(GTK_FONT_BUTTON
                                (if_preferences_get_widget
                                 ("cmdFont")), options_get_str(opt, 
                                                                "font-family"));

  altEditor = options_get_str(opt, "editor_alternative");

  if (strcmp(altEditor, "0") == 0) {
    altEditor = NULL;
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                                 (if_preferences_get_widget
                                  ("chkPrefAltEditor")), TRUE);
    gtk_entry_set_text(GTK_ENTRY(if_preferences_get_widget("entryPrefEditor")),
                       altEditor);
  }

  if (options_get_int(opt, "background_transparent")) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
                                 if_preferences_get_widget("chkTransparentBg")),
                                 TRUE);
    gtk_widget_set_sensitive(GTK_WIDGET(
                             if_preferences_get_widget("hscaleTransparancy")),
                             TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
                                 if_preferences_get_widget("chkTransparentBg")),
                                 FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(
                             if_preferences_get_widget("hscaleTransparancy")),
                             FALSE);
  }
  
  gtk_range_set_value(GTK_RANGE(if_preferences_get_widget(
                                    "hscaleTransparancy")), 
                          options_get_int(opt, "background_transparancy"));
  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                               (if_preferences_get_widget
                                ("chkPrefEmbedEditor")), 
                                options_get_int(opt, "editor_embed"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                               (if_preferences_get_widget
                                ("chkPrefNeedsTerminal")), 
                                options_get_int(opt, "editor_needs_terminal"));

  gtk_widget_set_sensitive(if_preferences_get_widget("entryPrefEditor"),
                           altEditor != NULL);
  gtk_widget_set_sensitive(if_preferences_get_widget("chkPrefEmbedEditor"),
                           altEditor != NULL);
  gtk_widget_set_sensitive(if_preferences_get_widget("chkPrefNeedsTerminal"),
                           altEditor != NULL
                           && options_get_int(opt, "editor_embed") == 0);

  glade_xml_signal_connect(xmlPreferences, "on_chkPrefAltEditor_clicked",
                           G_CALLBACK(on_chkPrefAltEditor_clicked));
  glade_xml_signal_connect(xmlPreferences, "on_chkPrefEmbedEditor_clicked",
                           G_CALLBACK(on_chkPrefEmbedEditor_clicked));
  glade_xml_signal_connect(xmlPreferences, "on_chkForeground_clicked",
                           G_CALLBACK(on_chkForeground_clicked));
  glade_xml_signal_connect(xmlPreferences, "on_chkBackground_clicked",
                           G_CALLBACK(on_chkBackground_clicked));

  glade_xml_signal_connect(xmlPreferences, "on_chkTransparentBg_clicked",
                           G_CALLBACK(on_chkTransparentBg_clicked));

  glade_xml_signal_connect_data(xmlPreferences, "on_cmdBold_clicked",
                                G_CALLBACK(on_cmdStyle_clicked), 
                                GINT_TO_POINTER(1));
  glade_xml_signal_connect_data(xmlPreferences, "on_cmdItalic_clicked",
                                G_CALLBACK(on_cmdStyle_clicked), 
                                GINT_TO_POINTER(2));
  glade_xml_signal_connect_data(xmlPreferences, "on_cmdUnderline_clicked",
                                G_CALLBACK(on_cmdStyle_clicked), 
                                GINT_TO_POINTER(3));
  glade_xml_signal_connect_data(xmlPreferences, "on_cmdStrikethrough_clicked",
                                G_CALLBACK(on_cmdStyle_clicked), 
                                GINT_TO_POINTER(4));
  glade_xml_signal_connect(xmlPreferences, "on_clbForeground_color_set",
                           G_CALLBACK(on_clbForeground_color_set));
  glade_xml_signal_connect(xmlPreferences, "on_clbBackground_color_set",
                           G_CALLBACK(on_clbBackground_color_set));

  return if_preferences_get_widget("dlgPreferences");
}

void preferences_load_colors() {
  int i;
  GtkColorButton *but;
  GdkColor col;
  
  for (i = 0; i < sizeof(color_mapping) / sizeof(keyvaluepair); i++) {
    but = GTK_COLOR_BUTTON(if_preferences_get_widget(color_mapping[i].key));
    
    gdk_color_parse(options_get_str(prefs, color_mapping[i].value), &col);
    gtk_color_button_set_color(but, &col);
  }

}

void
preferences_save_colors() {
  int i;
  GtkColorButton *but;
  GdkColor col;
  gchar *colStr;
  
  for (i = 0; i < sizeof(color_mapping) / sizeof(keyvaluepair); i++) {
    but = GTK_COLOR_BUTTON(if_preferences_get_widget(color_mapping[i].key));
    
    gtk_color_button_get_color(but, &col);
    colStr = g_strdup_printf("#%04X%04X%04X", col.red, col.green, col.blue);
    
    options_set_str(prefs, color_mapping[i].value, colStr);
    g_free(colStr);
  }
}

void
preferences_save_editor_color(GtkTreeModel *model, GtkTreeIter *iter) {
  gchar *name, *id, *optKey, *optValue, *fg, *bg;
  GtkSourceTagStyle *style;
  
  gtk_tree_model_get(model, iter, EDITOR_NAME, &name, EDITOR_STYLE, &style,
                     EDITOR_ID, &id, -1);
  
  optKey = g_strconcat("editor_", name, NULL);
  
  if (!(style->mask & GTK_SOURCE_TAG_STYLE_USE_FOREGROUND)) {
    fg = g_strdup("0");
  } else {
    fg = g_strdup_printf("#%04X%04X%04X", style->foreground.red, 
                         style->foreground.green, style->foreground.blue);
  }
  
  if (!(style->mask & GTK_SOURCE_TAG_STYLE_USE_BACKGROUND)) {
    bg = g_strdup("0");
  } else {
    bg = g_strdup_printf("#%04X%04X%04X", style->background.red, 
                         style->background.green, style->background.blue);
  }

  optValue = g_strdup_printf("%s,%s,%d,%d,%d,%d", fg, bg, style->bold,
                             style->italic, style->underline, 
                             style->strikethrough);

  debug_msg(1, "setting editor color setting %s to %s", optKey, optValue);
  options_set_str(prefs, optKey, optValue);
  gtk_source_language_set_tag_style(editor_get_language(), id, style);
  
  g_free(optKey);
  g_free(optValue);
  g_free(id);
  g_free(fg);
  g_free(bg);
  g_free(name);
  //gtk_source_tag_style_free(style);
}

void
preferences_save_editor_colors() {
   GtkTreeIter iter;
   GtkTreeView *vw = GTK_TREE_VIEW(if_preferences_get_widget("tvEditorColors"));
   GtkTreeModelSort *model = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(vw));
   GtkTreeModel *store = gtk_tree_model_sort_get_model(model);
   
   if (gtk_tree_model_get_iter_first(store, &iter)) {
     preferences_save_editor_color(store, &iter);
     
     while (gtk_tree_model_iter_next(store, &iter)) {
       preferences_save_editor_color(store, &iter);
     }
   }
}

gboolean
check_preferences_values() {
  const gchar *altEditor =
    gtk_entry_get_text(GTK_ENTRY
                       (if_preferences_get_widget("entryPrefEditor")));
  const gchar *fontName = 
    gtk_font_button_get_font_name(GTK_FONT_BUTTON
                                (if_preferences_get_widget
                                 ("cmdFont")));
  gboolean useAltEditor =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                 (if_preferences_get_widget
                                  ("chkPrefAltEditor")));

  if (useAltEditor && strlen(altEditor) == 0) {
    GtkWidget *mDlg =
      gtk_message_dialog_new(GTK_WINDOW
                             (if_preferences_get_widget("dlgPreferences")),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                             "Editor can't be \"\", enter a non-null command!");
    gtk_dialog_run(GTK_DIALOG(mDlg));
    gtk_widget_destroy(mDlg);
    notebook_focus_with_label(GTK_NOTEBOOK
                              (if_preferences_get_widget("notePreferences")),
                              _("editor"));
    gtk_widget_grab_focus(if_preferences_get_widget("entryPrefEditor"));

    return FALSE;
  }

  if (useAltEditor) {
    options_set_str(prefs, "editor_alternative", (gchar *) altEditor);
  } else {
    options_set_int(prefs, "editor_alternative", 0);
  }

  options_set_int(prefs, "editor_embed",
                  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                               (if_preferences_get_widget
                                                ("chkPrefEmbedEditor"))));
  options_set_int(prefs, "editor_needs_terminal",
                  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                               (if_preferences_get_widget
                                                ("chkPrefNeedsTerminal"))));

  options_set_str(prefs, "font-family", (gchar *)fontName);
  
  options_set_int(prefs, "bold-colors",
                  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                               (if_preferences_get_widget
                                                ("chkBold"))));
                                                
  preferences_save_colors();
  
  options_set_int(prefs, "background_transparent",
                  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                               (if_preferences_get_widget
                                                ("chkTransparentBg"))));

  options_set_int(prefs, "background_transparancy",
                  gtk_range_get_value(GTK_RANGE
                                      (if_preferences_get_widget
                                       ("hscaleTransparancy"))));
  
  preferences_save_editor_colors();
  
  return TRUE;
}

gboolean
create_preferences(options * opt) {
  GtkWidget *dlg = create_dlgPreferences(opt);
  gint result;
  gboolean isOkay = FALSE;
  GList *elem;
  world *wld;
  
  while (!isOkay) {
    isOkay = TRUE;
    result = gtk_dialog_run(GTK_DIALOG(dlg));

    switch (result) {
    case GTK_RESPONSE_OK:
    case GTK_RESPONSE_APPLY:
      if (check_preferences_values()) {
        options_save(opt);
        
        for (elem = world_get_worlds(); elem; elem = elem->next) {
          wld = (world *)(elem->data);
      
          if (wld->loaded) {
            world_apply_ansi(wld);
          }
        }
        
        if (result == GTK_RESPONSE_APPLY) {
          isOkay = FALSE;
        }
      } else {
        isOkay = FALSE;
      }

      break;
    default:
      break;
    }
  }

  gtk_widget_destroy(dlg);
  
  if (result == GTK_RESPONSE_OK) {

  }
  
  return (result == GTK_RESPONSE_OK);
}

/* CALLBACKS */

void
on_chkPrefEmbedEditor_clicked(GtkButton * button, gpointer user_data) {
  gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));

  if (active) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                                 (if_preferences_get_widget
                                  ("chkPrefNeedsTerminal")), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                                 (if_preferences_get_widget
                                  ("chkPrefNeedsTerminal")),
                                 options_get_int(prefs,
                                                 "editor_needs_terminal"));
  }

  gtk_widget_set_sensitive(if_preferences_get_widget("chkPrefNeedsTerminal"),
                           !active);
}

void
on_chkPrefAltEditor_clicked(GtkButton * button, gpointer user_data) {
  gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));

  if (!active) {
    gtk_entry_set_text(GTK_ENTRY(if_preferences_get_widget("entryPrefEditor")),
                       "");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                                 (if_preferences_get_widget
                                  ("chkPrefEmbedEditor")), FALSE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                                 (if_preferences_get_widget
                                  ("chkPrefNeedsTerminal")), FALSE);
  }

  gtk_widget_set_sensitive(if_preferences_get_widget("entryPrefEditor"),
                           active);
  gtk_widget_set_sensitive(if_preferences_get_widget("chkPrefEmbedEditor"),
                           active);
  gtk_widget_set_sensitive(if_preferences_get_widget("chkPrefNeedsTerminal"),
                           active);
}

void
editor_style_do_enable() {
  gtk_widget_set_sensitive(if_preferences_get_widget("chkForeground"), TRUE);
  gtk_widget_set_sensitive(if_preferences_get_widget("chkBackground"), TRUE);
  gtk_widget_set_sensitive(if_preferences_get_widget("cmdBold"), TRUE);
  gtk_widget_set_sensitive(if_preferences_get_widget("cmdItalic"), TRUE);
  gtk_widget_set_sensitive(if_preferences_get_widget("cmdUnderline"), TRUE);
  gtk_widget_set_sensitive(if_preferences_get_widget("cmdStrikethrough"), TRUE);
}

void 
on_editor_color_row_changed(GtkTreeSelection *selection, gpointer data) {
  GtkTreeIter iter;
  GtkTreeModel *model;
  GtkSourceTagStyle *style;
  GtkColorButton *fg, *bg;
  GtkToggleButton *fgt, *bgt;
  GdkColor emptyCol;
  
  gdk_color_parse("black", &emptyCol);
  
  gtk_tree_selection_get_selected(selection, &model, &iter);
  gtk_tree_model_get(model, &iter, EDITOR_STYLE, &style, -1);
  
  fg = GTK_COLOR_BUTTON(if_preferences_get_widget("clbForeground"));  
  bg = GTK_COLOR_BUTTON(if_preferences_get_widget("clbBackground"));  
  fgt = GTK_TOGGLE_BUTTON(if_preferences_get_widget("chkForeground"));  
  bgt = GTK_TOGGLE_BUTTON(if_preferences_get_widget("chkBackground"));  
  
  if (style->mask & GTK_SOURCE_TAG_STYLE_USE_FOREGROUND) {
    gtk_color_button_set_color(fg, &(style->foreground));
    gtk_widget_set_sensitive(GTK_WIDGET(fg), TRUE);
    gtk_toggle_button_set_active(fgt, TRUE);
  } else {
    gtk_color_button_set_color(fg, &emptyCol);
    gtk_widget_set_sensitive(GTK_WIDGET(fg), FALSE);
    gtk_toggle_button_set_active(fgt, FALSE);  
  }

  if (style->mask & GTK_SOURCE_TAG_STYLE_USE_BACKGROUND) {
    gtk_color_button_set_color(bg, &(style->background));
    gtk_widget_set_sensitive(GTK_WIDGET(bg), TRUE);
    gtk_toggle_button_set_active(bgt, TRUE);
  } else {
    gtk_color_button_set_color(bg, &emptyCol);
    gtk_widget_set_sensitive(GTK_WIDGET(bg), FALSE);
    gtk_toggle_button_set_active(bgt, FALSE);
  }
 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_preferences_get_widget(
                               "cmdBold")), style->bold);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_preferences_get_widget(
                               "cmdItalic")), style->italic);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_preferences_get_widget(
                               "cmdUnderline")), style->underline);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_preferences_get_widget(
                               "cmdStrikethrough")), style->strikethrough);
  editor_style_do_enable();
}

typedef struct _selection_info {
  GtkTreeIter siter;
  GtkTreeIter iter;
  GtkListStore *store;
  GtkTreeModel *model;
  GtkSourceTagStyle *style;  
} selection_info;

selection_info
get_selection_info() {
  GtkTreeSelection *selection;
  selection_info info;
  
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(
                 if_preferences_get_widget("tvEditorColors")));
                 
  gtk_tree_selection_get_selected(selection, &(info.model), &(info.siter));
  gtk_tree_model_get(info.model, &(info.siter), 
                     EDITOR_STYLE, &(info.style), -1);
  gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(
                                                 info.model), 
                                                 &(info.iter), &(info.siter));
                                                 
  info.store = GTK_LIST_STORE(gtk_tree_model_sort_get_model(
                              GTK_TREE_MODEL_SORT(info.model)));
  
  return info;
}

void
on_chkForeground_clicked(GtkButton * button, gpointer user_data) {
  selection_info info;
  gboolean tog = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
  GtkWidget *clb = if_preferences_get_widget("clbForeground");
  GdkColor emptyCol;
  
  info = get_selection_info();
  gdk_color_parse("black", &emptyCol);
  
  if (tog) {
    info.style->mask = info.style->mask | GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;
  } else {
    info.style->mask = info.style->mask & ~GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;
    gtk_color_button_set_color(GTK_COLOR_BUTTON(clb), &emptyCol);
    info.style->foreground = emptyCol;
  }
  
  gtk_list_store_set(info.store, &(info.iter), EDITOR_STYLE, info.style, -1);
  gtk_source_tag_style_free(info.style);
  
  gtk_widget_set_sensitive(clb, tog);
}

void
on_chkBackground_clicked(GtkButton * button, gpointer user_data) {
  selection_info info;
  gboolean tog = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
  GtkWidget *clb = if_preferences_get_widget("clbBackground");
  GdkColor emptyCol;
  
  info = get_selection_info();
  gdk_color_parse("black", &emptyCol);
  
  if (tog) {
    info.style->mask = info.style->mask | GTK_SOURCE_TAG_STYLE_USE_BACKGROUND;
  } else {
    info.style->mask = info.style->mask & ~GTK_SOURCE_TAG_STYLE_USE_BACKGROUND;
    gtk_color_button_set_color(GTK_COLOR_BUTTON(clb), &emptyCol);
    info.style->background = emptyCol;
  }
  
  gtk_list_store_set(info.store, &(info.iter), EDITOR_STYLE, info.style, -1);
  gtk_source_tag_style_free(info.style);
  
  gtk_widget_set_sensitive(clb, tog);
}

void on_cmdStyle_clicked(GtkButton * button, gpointer user_data) {
  selection_info info;
  int type = GPOINTER_TO_INT(user_data);
  gboolean tog = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
  
  info = get_selection_info();
  
  switch (type) {
    case 1: /* bold */
      info.style->bold = tog;
      break;
    case 2: /* italic */
      info.style->italic = tog;
      break;
    case 3: /* underline */
      info.style->underline = tog;
      break;
    case 4:
      info.style->strikethrough = tog;
      break;
  }

  gtk_list_store_set(info.store, &(info.iter), EDITOR_STYLE, info.style, -1);
  gtk_source_tag_style_free(info.style);    
}

void
on_clbForeground_color_set(GtkColorButton *widget, gpointer user_data) {
  selection_info info;
  
  info = get_selection_info();
  gtk_color_button_get_color(widget, &(info.style->foreground));
  
  gtk_list_store_set(info.store, &(info.iter), EDITOR_STYLE, info.style, -1);
  gtk_source_tag_style_free(info.style);
}

void
on_clbBackground_color_set(GtkColorButton *widget, gpointer user_data) {
  selection_info info;
  
  info = get_selection_info();
  gtk_color_button_get_color(widget, &(info.style->background));
  
  gtk_list_store_set(info.store, &(info.iter), EDITOR_STYLE, info.style, -1);
  gtk_source_tag_style_free(info.style);
}

void 
on_chkTransparentBg_clicked(GtkButton * button, gpointer user_data) {
  gboolean ac = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
  
  gtk_widget_set_sensitive(GTK_WIDGET(
                           if_preferences_get_widget("hscaleTransparancy")),
                           ac);
}
