#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HASRUBY
#  include <gtk/gtk.h>
#  include <string.h>
#  include <stdio.h>
#  include <errno.h>
#  include <unistd.h>
#  include <sys/types.h>

#  include <gtksourceview/gtksourceview.h>
#  include <gtksourceview/gtksourcelanguage.h>
#  include <gtksourceview/gtksourcelanguagesmanager.h>
#  include <gtksourceview/gtksourcetag.h>

#  include <libgnomevfs/gnome-vfs.h>

#  include "if_main.h"
#  include "if_scripts.h"
#  include "world.h"
#  include "debug.h"
#  include "main.h"
#  include "support.h"
#  include "script.h"

  typedef enum _scripts_columns {
    SCRIPTS_NAME,
    SCRIPTS_N
  } scripts_columns;

  typedef struct _sel_info {
    GtkTreeView *view;
    GtkTreeModel *model;
    GtkTreeIter seliter;
    GtkTreeIter parent;
    gboolean hasParent;
    gboolean canSave;
    gchar *filename;  
  } sel_info;
  
  static GtkTextTagTable *scriptsTagTable;
  static GladeXML *scriptsXML = NULL;
  static GtkTextBuffer *scriptsConsoleBuf = NULL;
  static gchar *curEdit = NULL;
  static GtkTextBuffer *editorBuf;
  
  gboolean on_wndScripts_delete(GtkWidget *widget, GdkEvent *event,
                                    gpointer user_data);

  void on_files_changed(GtkTreeSelection *treeselection, gpointer user_data);
  void on_tvwFiles_row_activated(GtkTreeView *treeview, GtkTreePath *arg1,
                                 GtkTreeViewColumn *arg2, gpointer user_data);
  void on_editorBuffer_changed(GtkTextBuffer *buffer, gpointer user_data);

  void on_tlbCmdNew_clicked(GtkToolButton *button, gpointer user_data);
  void on_tlbCmdSave_clicked(GtkToolButton *button, gpointer user_data);
  void on_tlbCmdSaveAs_clicked(GtkToolButton *button, gpointer user_data);
  void on_tlbCmdDelete_clicked(GtkToolButton *button, gpointer user_data);
  
  GtkWidget *
  if_scripts_get_widget(gchar *name) {
    return glade_xml_get_widget(scriptsXML, name);
  }

  gboolean
  if_scripts_can_write(gchar *filename) {
    GnomeVFSFileInfo info;
    
    gnome_vfs_get_file_info(filename, &info, 
                            GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS);
    
    if (info.permissions & GNOME_VFS_PERM_OTHER_WRITE) {
      return TRUE;
    } else if (geteuid() == info.uid && 
               (info.permissions & GNOME_VFS_PERM_USER_WRITE)) {
      return TRUE;
    } else if (getegid() == info.gid &&
               (info.permissions & GNOME_VFS_PERM_GROUP_WRITE)) {
      return TRUE;
    } else {
      return FALSE;
    }
  }
  
  void
  if_scripts_set_status(gchar *msg) {
    gtk_statusbar_pop(GTK_STATUSBAR(if_scripts_get_widget("stbScriptEditor")),
                           0);
    gtk_statusbar_push(GTK_STATUSBAR(if_scripts_get_widget("stbScriptEditor")),
                           0, msg);
  }
  
  gboolean
  if_scripts_editor_save(gchar *filename) {
    FILE *f;
    gchar *msg, *text;
    GtkTextIter start, end;
    
    if (!filename) {
      return FALSE;
    }
    
    f = fopen(filename, "w");
    
    if (f) {
      gtk_text_buffer_get_start_iter(editorBuf, &start);
      gtk_text_buffer_get_end_iter(editorBuf, &end);
      text = gtk_text_buffer_get_text(editorBuf, &start, &end, TRUE);
      
      fputs(text, f);
      
      msg = g_strconcat("Saved ", filename, NULL);  
      if_scripts_set_status(msg);
      g_free(msg);
      
      if (curEdit && filename != curEdit) {
        if (curEdit) {
          g_free(curEdit);
        }
      
        curEdit = g_strdup(filename);
      }
      
      fclose(f);
      return TRUE;
    } else {
      msg = g_strconcat("Saving failed: ", strerror(errno), NULL);
      gnoemoe_error_dialog(msg, GTK_WINDOW(if_scripts_get_widget("wndScripts")));
      g_free(msg);
      
      return FALSE;
    }
  }
  
  void
  if_scripts_editor_load(sel_info *info) {
    FILE *f;
    gchar line[1024];
    GtkTextIter end;
    gchar *lline, *msg;
    
    gtk_text_buffer_set_text(editorBuf, "", 0);
    gtk_text_buffer_get_end_iter(editorBuf, &end);

    if (info) {
      f = fopen(info->filename, "r");

      if (f) {
        while (fgets((char *) &line, 1024 - 1, f) != NULL) {
          lline = g_locale_to_utf8((gchar *)&line, strlen((char *)&line), NULL, NULL, NULL);
          
          if (lline) {
            gtk_text_buffer_insert(editorBuf, &end, lline, strlen(lline));
            g_free(lline);
          } else {
            gtk_text_buffer_insert(editorBuf, &end, (gchar *)&line, strlen((char *)&line));        
          }
        }

        fclose(f);

        gtk_widget_set_sensitive(if_scripts_get_widget("tlbCmdSave"), info->canSave);

        if (curEdit) {
          g_free(curEdit);
          curEdit = NULL;
        }
        
        curEdit = g_strdup(info->filename);
        msg = g_strconcat("Loaded ", info->filename, NULL);
        if_scripts_set_status(msg);
        g_free(msg);
      } else {
        debug_msg(1, "if_scripts_editor_load: file (%s) could not be read: %s",
                curEdit, strerror(errno));
      }
    } else {
      if (curEdit) {
        g_free(curEdit);
        curEdit = NULL;
      }
      
      if_scripts_set_status("New <untitled>");
    }
  }

  void
  if_scripts_init_console() {
    GtkTextView *txt = GTK_TEXT_VIEW(if_scripts_get_widget("txtVwScriptsConsole"));
    GdkColor bg;
    PangoFontDescription *f;

    gtk_text_view_set_buffer(txt, scriptsConsoleBuf);
    gtk_text_view_set_editable(txt, FALSE);
    gdk_color_parse("black", &bg);
    gtk_widget_modify_base(GTK_WIDGET(txt), GTK_STATE_NORMAL, &bg);
    
    f =
      pango_font_description_from_string("monospace 8");
    
    if (f) {
      gtk_widget_modify_font(GTK_WIDGET(txt), f);
      pango_font_description_free(f);
    }
  }

  void
  if_scripts_tree_add_function(GtkTreeStore *model, GtkTreeIter *parent, 
                               script_function *fi) {
    GtkTreeIter item;
    gchar *name, *description, *all;
    
    gtk_tree_store_append(model, &item, parent);
    name = g_filename_to_utf8(fi->name, strlen(fi->name), NULL,
                                  NULL, NULL);
    description = g_locale_to_utf8(fi->description, strlen(fi->description),
                                   NULL, NULL, NULL);

    all = g_strconcat(name, " :: ", description, NULL);                                   
    gtk_tree_store_set(model, &item, SCRIPTS_NAME, all, -1);
    
    g_free(all);
    g_free(description);
    g_free(name);
  }

  void
  if_scripts_build_tree() {
    GtkTreeView *view = GTK_TREE_VIEW(if_scripts_get_widget("tvwScripts"));
    GtkTreeStore *model = GTK_TREE_STORE(gtk_tree_view_get_model(view));
    gchar *prev = NULL;
    GList *scripts;
    script_function *fi;
    GtkTreeIter parent, item;
    gchar *name;
    
    gtk_tree_store_clear(model);
    
    for (scripts = script_get_scripts(); scripts; scripts = scripts->next) {
      fi = (script_function *)(scripts->data);
      
      if (prev && strcmp(fi->filename, prev) == 0) {
        if_scripts_tree_add_function(model, &parent, fi);
      } else {
        gtk_tree_store_append(model, &item, NULL);
        name = g_filename_to_utf8(fi->filename, strlen(fi->filename), NULL,
                                  NULL, NULL);
        gtk_tree_store_set(model, &item, SCRIPTS_NAME, name, -1);
        g_free(name);
        prev = fi->filename;
        parent = item;
        if_scripts_tree_add_function(model, &parent, fi);
      }
    }  
  }

  void
  if_scripts_init_tree() {
    GtkTreeView *view = GTK_TREE_VIEW(if_scripts_get_widget("tvwScripts"));
    GtkTreeStore *model = gtk_tree_store_new(SCRIPTS_N, G_TYPE_STRING);
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    
    renderer = gtk_cell_renderer_text_new();
    column =
      gtk_tree_view_column_new_with_attributes(_("Scripts"), renderer, "text",
                                               SCRIPTS_NAME, NULL);
                                             
    gtk_tree_view_append_column(view, column);     
    
    gtk_tree_view_set_model(view, GTK_TREE_MODEL(model));
    
    if_scripts_build_tree();    
  }

  gboolean 
  if_scripts_add_file(gchar *filename) {
    GtkTreeView *view = GTK_TREE_VIEW(if_scripts_get_widget("tvwFiles"));
    GtkTreeStore *model = GTK_TREE_STORE(gtk_tree_view_get_model(view));
    GtkTreeIter share, home, parent, item;
    gchar *sharedir = g_strdup(PACKAGE_DATA_DIR "/" PACKAGE "/scripts");
    gchar *homedir = g_strconcat(main_get_app_path(), "/scripts", NULL);
    gchar *name;
     
    gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(model), &share, "0"); 
    gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(model), &home, "1");
     
    name = g_filename_to_utf8(filename, strlen(filename), NULL,
                                  NULL, NULL);
    if (strncmp(sharedir, name, strlen(sharedir)) == 0) {
      parent = share;
    } else if (strncmp(homedir, name, strlen(homedir)) == 0) {
      parent = home;
    } else {
      g_free(sharedir);
      g_free(homedir);
      g_free(name);
      return FALSE;
    }

    gtk_tree_store_append(model, &item, &parent);
    gtk_tree_store_set(model, &item, SCRIPTS_NAME, g_strrstr(name, "/") + 1, -1);
    
    g_free(name);
    g_free(sharedir);
    g_free(homedir);
    
    return TRUE;
  }

  void
  if_scripts_build_files() {
    GtkTreeView *view = GTK_TREE_VIEW(if_scripts_get_widget("tvwFiles"));
    GtkTreeStore *model = GTK_TREE_STORE(gtk_tree_view_get_model(view));
    GtkTreeIter item, share, home, parent;
    GtkTreePath *path;
    gchar *filename, *sharedir;
    gchar *name;
    GList *scripts;
    
    sharedir = g_strdup(PACKAGE_DATA_DIR "/" PACKAGE "/scripts");
    
    gtk_tree_store_clear(model);
    
    gtk_tree_store_append(model, &share, NULL);
    gtk_tree_store_set(model, &share, SCRIPTS_NAME, "SHARE", -1);
    
    gtk_tree_store_append(model, &home, NULL);
    gtk_tree_store_set(model, &home, SCRIPTS_NAME, "USER", -1);
        
    for (scripts = script_get_files(); scripts; scripts = scripts->next) {
      filename = (gchar *)(scripts->data);
      name = g_filename_to_utf8(filename, strlen(filename), NULL,
                                  NULL, NULL);
      if (strncmp(sharedir, name, strlen(sharedir)) == 0) {
        parent = share;
      } else {
        parent = home;
      }
      
      gtk_tree_store_append(model, &item, &parent);
      gtk_tree_store_set(model, &item, SCRIPTS_NAME, g_strrstr(name, "/") + 1, -1);
      
      g_free(name);
    }
    
    path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), &home);
    gtk_tree_view_expand_to_path(view, path);
    gtk_tree_path_free(path);

    path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), &share);
    gtk_tree_view_expand_to_path(view, path);
    gtk_tree_path_free(path);
    
    g_free(sharedir);
  }
  
  void 
  if_scripts_init_files() {
    GtkTreeView *view = GTK_TREE_VIEW(if_scripts_get_widget("tvwFiles"));
    GtkTreeStore *model = gtk_tree_store_new(SCRIPTS_N, G_TYPE_STRING);
    GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
    GtkTreeSelection *selection;
    GtkTreeViewColumn *column =
      gtk_tree_view_column_new_with_attributes(_("Filename"), renderer, "text",
                                               SCRIPTS_NAME, NULL);
                                             
    gtk_tree_view_append_column(view, column);    
    gtk_tree_view_set_model(view, GTK_TREE_MODEL(model));
    
    selection = gtk_tree_view_get_selection(view);
    
    g_signal_connect((gpointer)selection, "changed", 
                      G_CALLBACK(on_files_changed), NULL);
    if_scripts_build_files();   
  }

  GtkWidget *
  if_scripts_editor_init() {
    GtkSourceLanguagesManager *lm = gtk_source_languages_manager_new();
    GtkSourceLanguage *lang;
    GtkWidget *view;
    PangoFontDescription *f;
            
    lang =
    gtk_source_languages_manager_get_language_from_mime_type(lm, "text/x-ruby");

    view = gtk_source_view_new();
    editorBuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));

    g_signal_connect((gpointer)editorBuf, "changed", 
                     G_CALLBACK(on_editorBuffer_changed), NULL);

    if (lang) {
      gtk_source_buffer_set_language(GTK_SOURCE_BUFFER(editorBuf), lang);
    }

    f =
      pango_font_description_from_string(options_get_str
                                         (main_get_options(),
                                          "font-family"));
    gtk_widget_modify_font(view, f);

    gtk_source_view_set_insert_spaces_instead_of_tabs(GTK_SOURCE_VIEW(view),
                                                      TRUE);
    gtk_source_view_set_auto_indent(GTK_SOURCE_VIEW(view), TRUE);
    gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(view), TRUE);
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD_CHAR);
    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(view), 6);
    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(view), 6);

    gtk_source_buffer_set_highlight(GTK_SOURCE_BUFFER(editorBuf), TRUE);
    gtk_widget_show(view);

    return view;
  }

  GtkWidget *
  if_scripts_create() {
    GtkWidget *editor;
    
    if (scriptsXML != NULL) {
      return if_scripts_get_widget("wndScripts");
    }
    
    scriptsXML = glade_xml_new(G_XML, "wndScripts", NULL);
    
    if_scripts_init_console();
    if_scripts_init_tree();
    if_scripts_init_files();
    editor = if_scripts_editor_init();

    gtk_container_add(GTK_CONTAINER(if_scripts_get_widget("srlScriptEditor")), 
                      editor);
        
    glade_xml_signal_connect(scriptsXML, "on_wndScripts_delete", 
                             G_CALLBACK(on_wndScripts_delete));

    glade_xml_signal_connect(scriptsXML, "on_tvwFiles_row_activated",
                             G_CALLBACK(on_tvwFiles_row_activated));
                                                          
    glade_xml_signal_connect(scriptsXML, "on_tlbCmdNew_clicked", 
                             G_CALLBACK(on_tlbCmdNew_clicked));
    glade_xml_signal_connect(scriptsXML, "on_tlbCmdSave_clicked", 
                             G_CALLBACK(on_tlbCmdSave_clicked));
    glade_xml_signal_connect(scriptsXML, "on_tlbCmdSaveAs_clicked", 
                             G_CALLBACK(on_tlbCmdSaveAs_clicked));
    glade_xml_signal_connect(scriptsXML, "on_tlbCmdDelete_clicked", 
                             G_CALLBACK(on_tlbCmdDelete_clicked));
                             
    gtk_window_set_transient_for(GTK_WINDOW(if_scripts_get_widget("wndScripts")),
                                 GTK_WINDOW(if_main_get_widget("wndMain")));
    if_scripts_set_status("New <untitled>");
    return if_scripts_get_widget("wndScripts");
  }

  GtkTextTag *
  if_scripts_create_tag(gchar *name, gchar *fg) {
    GtkTextTag *tag;
    GdkColor col;
    
    gdk_color_parse(fg, &col);
    tag = gtk_text_tag_new(name);
    g_object_set(G_OBJECT(tag), "foreground-gdk", &col, NULL);
    
    return tag;
  }

  void 
  if_scripts_init() {
    
    scriptsTagTable = gtk_text_tag_table_new();
    
    gtk_text_tag_table_add(scriptsTagTable, if_scripts_create_tag("scripts-run", 
                                                              "#0000CCCC0000"));
    gtk_text_tag_table_add(scriptsTagTable, if_scripts_create_tag("scripts-msg", 
                                                              "gray"));
    gtk_text_tag_table_add(scriptsTagTable, if_scripts_create_tag("scripts-error",
                                                              "#FFFF00000000"));

    scriptsConsoleBuf = gtk_text_buffer_new(scriptsTagTable);
  }

  void 
  if_scripts_fini() {
    g_object_unref(scriptsTagTable);
    g_object_unref(scriptsConsoleBuf);
  }

  void if_scripts_add(scriptc_type mtype, gchar *msg) {
    gchar *m, *tagName = NULL, *newLine;
    gchar p[2];
    GtkTextIter end;
    
    p[1] = '\0';
    
    switch (mtype) {
      case SCRIPTC_RUN:
        p[0] = ':';
        tagName = g_strdup("scripts-run");
        break;
      case SCRIPTC_MSG:
        p[0] = '#'; 
        tagName = g_strdup("scripts-msg");
        break;
      case SCRIPTC_ERROR:
        p[0] = '!'; 
        tagName = g_strdup("scripts-error");
        break;
    }
    
    m = g_strconcat(p, msg, "\n", NULL);
    gtk_text_buffer_get_end_iter(scriptsConsoleBuf, &end);
    
    // convert to UTF-8
    newLine = g_locale_to_utf8(m, strlen(m), NULL, NULL, NULL);

    if (newLine == NULL) {  
      gtk_text_buffer_insert_with_tags_by_name(scriptsConsoleBuf, &end, 
                                     m, strlen(m), tagName, NULL);
    } else {
      gtk_text_buffer_insert_with_tags_by_name(scriptsConsoleBuf, &end, 
                                     newLine, strlen(newLine), tagName, NULL);
      g_free(newLine);
    }
    
    g_free(m);
    g_free(tagName);
  }

  gboolean
  if_scripts_selection_info(sel_info *info) {
    GtkTreeView *view = GTK_TREE_VIEW(if_scripts_get_widget("tvwFiles"));
    GtkTreeModel *model = gtk_tree_view_get_model(view);
    GtkTreeSelection *selection = gtk_tree_view_get_selection(view);
    gchar *parentName, *name;
    
    info->view = view;
    info->model = model;
    
    if (gtk_tree_selection_get_selected(selection, &model, &(info->seliter))) {
      info->hasParent = gtk_tree_model_iter_parent(model, &(info->parent), &(info->seliter));
      
      if (info->hasParent) {
        gtk_tree_model_get(model, &(info->parent), SCRIPTS_NAME, &parentName, -1);
        gtk_tree_model_get(model, &(info->seliter), SCRIPTS_NAME, &name, -1);
        
        if (strcmp(parentName, "SHARE") == 0) {
          info->filename = g_strconcat(PACKAGE_DATA_DIR "/" PACKAGE "/scripts/", 
                                 name, NULL);
        } else {
          info->filename = g_strconcat(main_get_app_path(), "/scripts/", name, NULL);
        }
        
        g_free(name);
        g_free(parentName);
        info->canSave = if_scripts_can_write(info->filename);
      } else {
        info->filename = NULL;
      }
      
      return TRUE;
    } else {
      return FALSE;
    }
  }

  void
  if_scripts_update() {
    if_scripts_build_tree();
    if_scripts_build_files();  
  }

  /* CALLBACKS */

  gboolean 
  on_wndScripts_delete(GtkWidget *widget, GdkEvent *event,
                           gpointer user_data) {
    scriptsXML = NULL;
    
    if (curEdit) {
      g_free(curEdit);
    }
    
    return FALSE;
  }
  
  void 
  on_files_changed(GtkTreeSelection *treeselection, gpointer user_data) {
    sel_info info;

    if (if_scripts_selection_info(&info)) {
      gtk_widget_set_sensitive(if_scripts_get_widget("tlbCmdDelete"), 
                             info.hasParent && info.canSave);
    
      if (info.filename) {
        g_free(info.filename);
      }
    }    
  }
  
  void 
  on_tvwFiles_row_activated(GtkTreeView *treeview, GtkTreePath *arg1,
                                 GtkTreeViewColumn *arg2, gpointer user_data) {
    sel_info si;
                         
    if (if_scripts_selection_info(&si) && si.filename) {
      if_scripts_editor_load(&si);
      
      if (si.filename) {
        g_free(si.filename);
      }
    }
  }
  
  void 
  on_editorBuffer_changed(GtkTextBuffer *buffer, gpointer user_data) {
    gchar *msg;
    
    if (curEdit) {
      msg = g_strconcat("Changed ", curEdit, NULL);
    } else {
      msg = g_strdup("Changed <untitled>");
    }
    
    if_scripts_set_status(msg);
    g_free(msg);  
  }
  
  void 
  on_tlbCmdNew_clicked(GtkToolButton *button, gpointer user_data) {
    if_scripts_editor_load(NULL);
  }
  
  void 
  on_tlbCmdSave_clicked(GtkToolButton *button, gpointer user_data) {
    if (curEdit) {
      if_scripts_editor_save(curEdit);
    } else {
      on_tlbCmdSaveAs_clicked(button, user_data);
    }
  }
  
  void 
  on_tlbCmdSaveAs_clicked(GtkToolButton *button, gpointer user_data) {
    GtkWidget *dlg;
    gchar *filename, *di;
    
    dlg = gtk_file_chooser_dialog_new ("Save File",
				      GTK_WINDOW(if_scripts_get_widget("wndScripts")),
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);
		
		if (curEdit) {
  		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dlg), curEdit);
    } else {
      di = g_strconcat(main_get_app_path(), "/scripts/untitled.rb", NULL);
      gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(dlg), di);
      gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dlg), "untitled.rb");
      g_free(di);
    }
				      		      
    if (gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_ACCEPT) {
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));

      if (if_scripts_editor_save(filename)) {
        if_scripts_add_file(filename);
      }
      
      g_free(filename);
    }
    
    gtk_widget_destroy(dlg);
  }
  
  void 
  on_tlbCmdDelete_clicked(GtkToolButton *button, gpointer user_data) {
    sel_info info;
    
    if (if_scripts_selection_info(&info)) {
      if (info.hasParent && info.canSave) {
        remove(info.filename);
        gtk_tree_store_remove(GTK_TREE_STORE(info.model), &(info.seliter));
      }
      
      g_free(info.filename);
    }
  }
#endif
