#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "if_world.h"
#include "gnoe_pixbuf.h"
#include "debug.h"
#include "main.h"
#include "if_main.h"
#include "net.h"
#include "g_options.h"
#include "support.h"
#include "world.h"

void on_tabExitButton_released(GtkButton * button, world * wld);
gboolean on_txtVwInput_key_pressed(GtkWidget * widget, GdkEventKey * event,
                                   world * wld);

void on_txtVwInput_begin(GtkTextBuffer * buffer, world * wld);
void on_txtVwInput_end(GtkTextBuffer * buffer, world * wld);
void on_noteWorld_switch_page(GtkNotebook * notebook, GtkNotebookPage * page,
                              guint page_num, world * wld);
void on_noteWorld_destroy(GtkWidget * caller, world * wld);
gboolean on_txtVwWorld_configure (GtkWidget *widget, GdkEventConfigure *event,
                                  world *wld);
                                 
// This function does NOT pack the label, it only creates it and returns it
GtkWidget *
create_tab_label(gchar * icon, gchar * caption, gboolean has_exit,
                 label_info * info) {
  // First create the gbox (size 3) which will contain an icon, a label and a exitbutton if has_exit is true
  GtkWidget *hboxTabLabel;

  hboxTabLabel = gtk_hbox_new(FALSE, 3);

  gtk_widget_show(hboxTabLabel);

  info->tabIcon = gtk_image_new_from_pixbuf(gnoe_pixbuf_get(icon));
  gtk_widget_set_size_request(info->tabIcon, 16, 16);
  gtk_widget_show(info->tabIcon);
  gtk_box_pack_start(GTK_BOX(hboxTabLabel), info->tabIcon, TRUE, TRUE, 0);

  info->tabLabel = gtk_label_new(caption);
  gtk_widget_show(info->tabLabel);
  gtk_box_pack_start(GTK_BOX(hboxTabLabel), info->tabLabel, FALSE, FALSE, 0);

  if (has_exit) {
    info->tabExitButton = gtk_button_new();
    gtk_widget_show(info->tabExitButton);

    gtk_box_pack_start(GTK_BOX(hboxTabLabel), info->tabExitButton, FALSE,
                       FALSE, 0);
    gtk_widget_set_size_request(info->tabExitButton, 16, 16);
    gtk_button_set_relief(GTK_BUTTON(info->tabExitButton), GTK_RELIEF_NONE);

    info->tabExitImage =
      gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(info->tabExitImage);
    gtk_container_add(GTK_CONTAINER(info->tabExitButton), info->tabExitImage);
  }

  return hboxTabLabel;
}

// This creates the world view, it does NOT pack it into the main notebook
GtkWidget *
create_world_view(world * wld) {
  GtkTextIter iter;
  
  // Create the xml file
  wld->widgets.xml = glade_xml_new(G_XML, "noteWorld", "wndContainer");

  wld->widgets.noteContainer =
    glade_xml_get_widget(wld->widgets.xml, "noteWorld");
  GTK_WIDGET_UNSET_FLAGS(wld->widgets.noteContainer, GTK_CAN_FOCUS);

  wld->widgets.hpane = glade_xml_get_widget(wld->widgets.xml, "hpaneWorld");

  wld->widgets.srlWndText =
    glade_xml_get_widget(wld->widgets.xml, "srlWndText");
  wld->widgets.txtVwWorld =
    glade_xml_get_widget(wld->widgets.xml, "txtVwWorld");
  wld->widgets.bufVwWorld =
    G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(wld->widgets.txtVwWorld)));
    
  GTK_WIDGET_UNSET_FLAGS(wld->widgets.txtVwWorld, GTK_CAN_FOCUS);
  gtk_widget_modify_base(wld->widgets.txtVwWorld, GTK_STATE_ACTIVE,
                         &(wld->widgets.txtVwWorld->style->
                           base[GTK_STATE_SELECTED]));

  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(wld->widgets.bufVwWorld),
                               &iter);
  gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(wld->widgets.bufVwWorld),
                              "end-of-buffer", &iter, FALSE);
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(wld->widgets.txtVwWorld),
                              GTK_WRAP_WORD_CHAR);

  wld->widgets.tvwUserlist =
    glade_xml_get_widget(wld->widgets.xml, "tvwUserlist");
  wld->widgets.srlWndUserlist =
    glade_xml_get_widget(wld->widgets.xml, "srlWndUserlist");
  gtk_widget_modify_base(wld->widgets.tvwUserlist, GTK_STATE_ACTIVE,
                         &(wld->widgets.tvwUserlist->style->
                           base[GTK_STATE_SELECTED]));

  wld->widgets.txtVwInput =
    glade_xml_get_widget(wld->widgets.xml, "txtVwInput");
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(wld->widgets.txtVwInput),
                              GTK_WRAP_WORD_CHAR);

  wld->widgets.stbStatus = glade_xml_get_widget(wld->widgets.xml, "stbStatus");

  gtk_statusbar_push(GTK_STATUSBAR(wld->widgets.stbStatus), 0,
                     _("Welcome to GnoeMoe, explorer of new worlds!"));

  wld->widgets.tlbStatusButtons =
    glade_xml_get_widget(wld->widgets.xml, "tlbStatusButtons");

  glade_xml_signal_connect_data(wld->widgets.xml, "on_txtVwInput_key_pressed",
                                G_CALLBACK(on_txtVwInput_key_pressed), wld);
  glade_xml_signal_connect_data(wld->widgets.xml, "on_noteWorld_switch_page",
                                G_CALLBACK(on_noteWorld_switch_page), wld);
  glade_xml_signal_connect_data(wld->widgets.xml, "on_noteWorld_destroy",
                                G_CALLBACK(on_noteWorld_destroy), wld);
  g_signal_connect((gpointer)(if_main_get_widget("wndMain")), "configure_event",
                                G_CALLBACK(on_txtVwWorld_configure), wld);
                                                                
  g_signal_connect((gpointer)gtk_text_view_get_buffer(GTK_TEXT_VIEW(
                   wld->widgets.txtVwInput)), "begin-user-action", 
                   G_CALLBACK(on_txtVwInput_begin), wld);
  g_signal_connect((gpointer)gtk_text_view_get_buffer(GTK_TEXT_VIEW(
                    wld->widgets.txtVwInput)), "end-user-action", 
                    G_CALLBACK(on_txtVwInput_end), wld);
  return wld->widgets.noteContainer;
}

  /* ##############################################################################
   * WORLD INTERFACE CALLBACKS
   * #-------------------------------------------------------------------------- */

void
on_noteWorld_destroy(GtkWidget * caller, world * wld) {

  options_set_int(wld->settings, "pane_position",
                  wld->widgets.hpane->allocation.width - 
                  gtk_paned_get_position(GTK_PANED(wld->widgets.hpane)));
  options_save(wld->settings);

  wld->widgets.noteContainer = NULL;
  wld->widgets.hpane = NULL;
  wld->widgets.srlWndText = NULL;
  wld->widgets.bufVwWorld = NULL;
  wld->widgets.txtVwWorld = NULL;
  wld->widgets.tvwUserlist = NULL;
  wld->widgets.srlWndUserlist = NULL;
  wld->widgets.txtVwInput = NULL;
  wld->widgets.stbStatus = NULL;
  wld->widgets.tlbStatusButtons = NULL;
}

void
on_tabExitButton_released(GtkButton * button, world * wld) {
  world_unload(wld);
}

gchar *
str_new_value(gchar * old, gchar * new) {
  if (old) {
    g_free(old);
  }
  if (new) {
    return g_strdup(new);
  } else {
    return NULL;
  }
}

void
txtVwInput_set_text(GtkTextBuffer * buf, gchar * text, int len) {
  GtkTextIter end;

  gtk_text_buffer_set_text(buf, text, len);
  gtk_text_buffer_get_end_iter(buf, &end);
  gtk_text_buffer_place_cursor(buf, &end);
}

void if_world_scroll_page(world *wld, int direction) {
  GtkScrolledWindow *srl = GTK_SCROLLED_WINDOW(wld->widgets.srlWndText);
  GtkAdjustment *adj = gtk_scrolled_window_get_vadjustment(srl);
  double newValue = adj->value + (adj->page_size * direction);
  
  if (newValue > adj->upper - adj->page_size) {
    gtk_adjustment_set_value(adj, adj->upper - adj->page_size);
  } else if (newValue < adj->lower) {
    gtk_adjustment_set_value(adj, adj->lower);
  } else {
    gtk_adjustment_set_value(adj, newValue);
  }
}

gboolean
on_txtVwInput_key_pressed(GtkWidget * widget, GdkEventKey * event, world * wld) {
  gchar *text;
  gboolean f = FALSE, result = TRUE;
  int i, buffer = options_get_int(wld->settings, "input_buffer");
  entry_info *inf = &(wld->entryBuffer);
  int written;
  GError *err = NULL;
  gchar *normal;
  GtkTextBuffer *buf;
  GtkTextIter start, end;

  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));

  gtk_text_buffer_get_bounds(buf, &start, &end);
  text = gtk_text_buffer_get_text(buf, &start, &end, FALSE);

  if (strlen(text) == 0
      && (inf->prevKey == GDK_Delete || inf->prevKey == GDK_BackSpace)) {
    inf->curPos = g_list_length(inf->buffer);
    inf->prefix = str_new_value(inf->prefix, NULL);
  }

  if (event->keyval == GDK_Up) {
    if (inf->curPos != 0) {
      if (inf->curPos == g_list_length(inf->buffer)) {
        inf->prefix = str_new_value(inf->prefix, text);
      }

      if (strlen(inf->prefix) == 0) {
        inf->curPos--;
        f = TRUE;
      } else {
        f = FALSE;

        for (i = inf->curPos - 1; i >= 0; i--) {
          if (strncmp
              ((gchar *) g_list_nth_data(inf->buffer, i), inf->prefix,
               strlen(inf->prefix)) == 0) {
            inf->curPos = i;
            f = TRUE;
            break;
          }
        }
      }

      if (f) {
        txtVwInput_set_text(buf,
                            (gchar *) g_list_nth_data(inf->buffer,
                                                      inf->curPos), -1);
      }
    }
  } else if (event->keyval == GDK_Down) {
    if (inf->curPos != g_list_length(inf->buffer)) {

      if (inf->curPos == g_list_length(inf->buffer) - 1) {
        inf->curPos = g_list_length(inf->buffer);
        txtVwInput_set_text(buf, inf->prefix, -1);
        return TRUE;
      }

      if (strlen(inf->prefix) == 0) {
        inf->curPos++;
        f = TRUE;
      } else {
        f = FALSE;

        for (i = inf->curPos + 1; i < g_list_length(inf->buffer); i++) {
          if (strncmp
              ((gchar *) g_list_nth_data(inf->buffer, i), inf->prefix,
               strlen(inf->prefix)) == 0) {
            f = TRUE;
            inf->curPos = i;
            break;
          }
        }
      }

      if (f) {
        txtVwInput_set_text(buf, g_list_nth_data(inf->buffer, inf->curPos),
                            -1);
      } else {
        txtVwInput_set_text(buf, inf->prefix, -1);
        inf->curPos = g_list_length(inf->buffer) - 1;
      }
    }
  } else if (event->keyval == GDK_Return && !(event->state & GDK_CONTROL_MASK)
             && !(event->state & GDK_SHIFT_MASK)) {
    if (strlen(text) > 0) {
      normal =
        g_convert_with_fallback(text, strlen(text),
                                options_get_str(wld->settings,
                                                "charset"), "UTF-8", "?", NULL,
                                &written, &err);

      if (normal) {
        world_process_input(wld, normal);
        g_free(normal);
      } else {
        debug_msg(1,
                  "on_txtVwInput_key_pressed: conversion failed! Sending UTF-8!!!!! (%s)",
                  err->message);
        world_process_input(wld, text);
        g_error_free(err);
      }

      while (g_list_length(inf->buffer) > buffer) {
        inf->buffer = g_list_remove(inf->buffer, inf->buffer->data);
      }

      inf->buffer = g_list_append(inf->buffer, g_strdup(text));
      inf->curPos = g_list_length(inf->buffer);

      txtVwInput_set_text(buf, "", 0);
    }
  } else if ((event->keyval == GDK_Page_Up || event->keyval == GDK_Page_Down) && 
             !(event->state & GDK_CONTROL_MASK && 
             !(event->state & GDK_SHIFT_MASK))) {
    if (event->keyval == GDK_Page_Up) {
      if_world_scroll_page(wld, -1);
    } else {
      if_world_scroll_page(wld, 1);
    }
  } else {
    result = FALSE;
  }

  g_free(text);
  return result;
}

void
on_noteWorld_switch_page(GtkNotebook * notebook, GtkNotebookPage * page,
                         guint page_num, world * wld) {
  if (page_num == 0) {
    world_activated(wld);
  }
}

void on_txtVwInput_begin(GtkTextBuffer * buffer, world * wld) {
  world_scroll_end_prepare(wld);
}

void on_txtVwInput_end(GtkTextBuffer * buffer, world * wld) {
  doEvents();
  world_scroll_end(wld);
}

gboolean on_txtVwWorld_configure (GtkWidget *widget, GdkEventConfigure *event,
                                  world *wld) {
  debug_msg(1, "updating background!");
  world_update_background(wld);
  return FALSE;                                 
}
