#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "if_main.h"
#include "if_world_properties.h"
#include "support.h"
#include "main.h"
#include "debug.h"
#include "g_options.h"
#include "world.h"

static GladeXML *xmlWorldProperties;
world *loadedWld;

world *
world_properties_get_world() {
  return loadedWld;
}

GtkWidget *
if_world_properties_get_widget(gchar * name) {
  return glade_xml_get_widget(xmlWorldProperties, name);
}

void
fill_charset_combo(GtkComboBox * cmb, gchar *select) {
  int i;
  gchar text[255];
  GtkListStore *store;

  store = gtk_list_store_new (1, G_TYPE_STRING);
  gtk_combo_box_set_model(cmb, GTK_TREE_MODEL(store));
  gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(cmb), 0);
               
  for (i = 0; i < sizeof(encodings) / sizeof(encoding); i++) {
    sprintf(text, "%s - %s", encodings[i].charset, encodings[i].name);
    gtk_combo_box_append_text(cmb, _(text));
    
    if (strcasecmp(encodings[i].charset, select) == 0) {
      gtk_entry_set_text(GTK_ENTRY(GTK_BIN(cmb)->child), text);
    }
  }
}

GtkWidget *
create_dlgWorld_properties(world * wld) {
  gchar *title;

  loadedWld = wld;

  xmlWorldProperties = glade_xml_new(G_XML, "dlgWorldProperties", NULL);

  title =
    g_strconcat(_("World properties - "),
                options_get_str(wld->settings, "name"), NULL);
  gtk_window_set_title(GTK_WINDOW
                       (if_world_properties_get_widget("dlgWorldProperties")),
                       title);
  g_free(title);

  gtk_entry_set_text(GTK_ENTRY(if_world_properties_get_widget("entryName")),
                     options_get_str(wld->settings, "name"));

  gtk_entry_set_text(GTK_ENTRY(if_world_properties_get_widget("entryHost")),
                     options_get_str(wld->settings, "host"));
  gtk_spin_button_set_value(GTK_SPIN_BUTTON
                            (if_world_properties_get_widget("spinPort")),
                            (double) (options_get_int(wld->settings, "port")));
  gtk_entry_set_text(GTK_ENTRY
                     (if_world_properties_get_widget("entryPlayerName")),
                     options_get_str(wld->settings, "player_name"));
  gtk_entry_set_text(GTK_ENTRY
                     (if_world_properties_get_widget("entryPassword")),
                     options_get_str(wld->settings, "password"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
                               (if_world_properties_get_widget
                                ("chkAutoreconnect")),
                               options_get_int(wld->settings, "reconnect"));
                               
  fill_charset_combo(GTK_COMBO_BOX(if_world_properties_get_widget(
                                   "cmbCharset")), options_get_str(
                                                    wld->settings, "charset"));

  return if_world_properties_get_widget("dlgWorldProperties");
}

gchar *
translate_charset(gchar * charset) {
  gchar *ch;
  int i;

  for (i = 0; i < sizeof(encodings) / sizeof(encoding); i++) {
    ch = g_strconcat(encodings[i].charset, " - ", encodings[i].name, NULL);

    if (g_strcasecmp(charset, ch) == 0) {
      g_free(ch);
      return g_strdup(encodings[i].charset);
    }

    g_free(ch);
  }

  return g_strdup(charset);
}

gboolean
check_world_properties_values() {

  GtkEntry *entry = GTK_ENTRY(if_world_properties_get_widget("entryName"));
  gchar *name = g_strdup(gtk_entry_get_text(entry));
  GtkEntry *entryHost = GTK_ENTRY(if_world_properties_get_widget("entryHost"));
  gchar *host = g_strdup(gtk_entry_get_text(entryHost));

  g_strstrip(name);
  g_strstrip(host);

  world *ifWld = world_get_by_name(name);
  world *newWld = world_properties_get_world();

  if (strlen(name) == 0) {
    GtkWidget *mDlg =
      gtk_message_dialog_new(GTK_WINDOW
                             (if_world_properties_get_widget
                              ("dlgWorldProperties")),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                             "Name can't be \"\", fill in a non-null name!");
    gtk_dialog_run(GTK_DIALOG(mDlg));
    gtk_widget_destroy(mDlg);
    notebook_focus_with_label(GTK_NOTEBOOK
                              (if_world_properties_get_widget
                               ("noteWorldProps")), _("general"));
    gtk_widget_grab_focus(GTK_WIDGET(entry));
    g_free(name);
    g_free(host);
    return FALSE;
  }


  if (ifWld != NULL && ifWld != newWld) {
    GtkWidget *mDlg =
      gtk_message_dialog_new(GTK_WINDOW
                             (if_world_properties_get_widget
                              ("dlgWorldProperties")),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                             _("Name can't be %s because it already exists!"),
                             name);
    gtk_dialog_run(GTK_DIALOG(mDlg));
    gtk_widget_destroy(mDlg);
    notebook_focus_with_label(GTK_NOTEBOOK
                              (if_world_properties_get_widget
                               ("noteWorldProps")), _("general"));
    gtk_widget_grab_focus(GTK_WIDGET(entry));
    g_free(name);
    g_free(host);
    return FALSE;
  }

  if (strlen(host) == 0) {
    GtkWidget *mDlg =
      gtk_message_dialog_new(GTK_WINDOW
                             (if_world_properties_get_widget
                              ("dlgWorldProperties")),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                             _
                             ("Host can't be \"\", fill in a non-null host!"));
    gtk_dialog_run(GTK_DIALOG(mDlg));
    gtk_widget_destroy(mDlg);
    notebook_focus_with_label(GTK_NOTEBOOK
                              (if_world_properties_get_widget
                               ("noteWorldProps")), _("server"));
    gtk_widget_grab_focus(GTK_WIDGET(entryHost));
    g_free(name);
    g_free(host);

    return FALSE;
  }

  options_set_str(newWld->settings, "name", name);

  options_set_str(newWld->settings, "host", host);
  options_set_int(newWld->settings, "port",
                  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON
                                                   (if_world_properties_get_widget
                                                    ("spinPort"))));
  options_set_int(newWld->settings, "reconnect",
                  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                               (if_world_properties_get_widget
                                                ("chkAutoreconnect"))));

  options_set_str(newWld->settings, "player_name",
                  (gchar *)
                  gtk_entry_get_text(GTK_ENTRY
                                     (if_world_properties_get_widget
                                      ("entryPlayerName"))));
  options_set_str(newWld->settings, "password",
                  (gchar *)
                  gtk_entry_get_text(GTK_ENTRY
                                     (if_world_properties_get_widget
                                      ("entryPassword"))));
                                      
  options_set_str(newWld->settings, "charset", 
                  translate_charset(
                    (gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_BIN(
                      if_world_properties_get_widget("cmbCharset"))->child))));
                                               

  g_free(name);
  g_free(host);

  return TRUE;
}

/* This function displays the world properties dialog and if the 
   response is ACCEPT then the path's are set to the possible new name
   also the world directory (wld->path) is moved to the new name
   the path to the options is also updated AND the options are saved.
   It DOES NOT update anything GUI'igs, so other functions have to do this!
   Return value is the response from the gtk_dialog_run... */
gint
create_world_properties(world * wld) {
  gchar *oldPath;
  GtkWidget *dlg = create_dlgWorld_properties(wld);
  gint result;
  gboolean isOkay = FALSE;

  if (wld->path != NULL) {
    oldPath = g_strdup(wld->path);
  } else {
    oldPath = NULL;
  }

  while (!isOkay) {
    isOkay = TRUE;
    result = gtk_dialog_run(GTK_DIALOG(dlg));

    switch (result) {
    case GTK_RESPONSE_OK:
      if (check_world_properties_values()) {
        if (wld->path != NULL) {
          g_free(wld->path);
        }

        wld->path =
          g_strconcat(main_get_worlds_path(), "/",
                      options_get_str(wld->settings, "name"), NULL);

        if (oldPath && g_file_test(oldPath, G_FILE_TEST_EXISTS)) {
          // Then! rename the oldPath to the possible newPath
          rename(oldPath, wld->path); // This could be rename(bla, bla); its really only usefull when the name has changed
        } else if (!g_file_test(wld->path, G_FILE_TEST_EXISTS)) {
          // There was no old path, so a new dir should be made
          mkdir(wld->path, 0755);
        }

        if (wld->settings->filename) {
          g_free(wld->settings->filename);
        }

        wld->settings->filename = g_strconcat(wld->path, "/settings", NULL);

        // Save it
        options_save(wld->settings);
      } else {
        isOkay = FALSE;
      }

      break;
    default:
      break;
    }
  }

  if (oldPath) {
    g_free(oldPath);
  }
  
  gtk_widget_destroy(dlg);
  return result;
}
