#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <libgnomeui/libgnomeui.h>

#include "support.h"

#include "debug.h"
#include "if_main.h"
#include "if_worlds_listing.h"
#include "if_world_properties.h"
#include "main.h"

void on_Autoload_toggled(GtkCellRendererToggle * cell, gchar * path_string,
                         gpointer user_data);
void on_cmdDeleteWorld_released(GtkButton * button, gpointer user_data);
void on_cmdNewWorld_released(GtkButton * button, gpointer user_data);
void on_cmdModifyWorld_released(GtkButton * button, gpointer user_data);
void on_cmdDuplicateWorld_released(GtkButton * button, gpointer user_data);
void on_cmdConnectWorld_released(GtkButton * button, gpointer user_data);
void on_tvWorlds_key_press(GtkWidget * widget, GdkEventKey * event,
                           gpointer user_data);

GladeXML *xmlWorldsListing;

GtkWidget *
if_worlds_listing_get_widget(gchar * name) {
  return glade_xml_get_widget(xmlWorldsListing, name);
}

void
add_tvWorldsItem(GtkListStore * store, gboolean autoload, char *name) {
  GtkTreeIter iter;

  gtk_list_store_append(store, &iter);
  gtk_list_store_set(store, &iter, AUTOLOAD_COLUMN, autoload, NAME_COLUMN,
                     name, -1);
}

GtkWidget *
create_dlgWorldsListing(void) {
  GtkWidget *tvWorlds;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkTreeModel *store;
  world *wld;
  GList *w;

  xmlWorldsListing = glade_xml_new(G_XML, "dlgWorldsListing", NULL);

  store =
    GTK_TREE_MODEL(gtk_list_store_new
                   (N_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING));
  tvWorlds = if_worlds_listing_get_widget("tvWorlds");
  gtk_tree_view_set_model(GTK_TREE_VIEW(tvWorlds), store);
  gtk_tree_selection_set_mode(gtk_tree_view_get_selection
                              (GTK_TREE_VIEW(tvWorlds)),
                              GTK_SELECTION_MULTIPLE);

  // Create autoload toggle column
  renderer = gtk_cell_renderer_toggle_new();
  g_object_set(G_OBJECT(renderer), "activatable", TRUE, NULL);
  g_signal_connect(renderer, "toggled", (GCallback) on_Autoload_toggled, NULL);

  column =
    gtk_tree_view_column_new_with_attributes(_("Autoload"), renderer, "active",
                                             AUTOLOAD_COLUMN, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tvWorlds), column);

  // Create world name column
  renderer = gtk_cell_renderer_text_new();

  column =
    gtk_tree_view_column_new_with_attributes(_("Name"), renderer, "text",
                                             NAME_COLUMN, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tvWorlds), column);

  // Okay, fill the rows with our worlds!
  for (w = world_get_worlds(); w; w = w->next) {
    wld = (world *) (w->data);
    add_tvWorldsItem(GTK_LIST_STORE(store),
                     options_get_int(wld->settings, "autoload"),
                     options_get_str(wld->settings, "name"));
  }

  glade_xml_signal_connect(xmlWorldsListing, "on_cmdDeleteWorld_released",
                           G_CALLBACK(on_cmdDeleteWorld_released));
  glade_xml_signal_connect(xmlWorldsListing, "on_cmdNewWorld_released",
                           G_CALLBACK(on_cmdNewWorld_released));
  glade_xml_signal_connect(xmlWorldsListing, "on_cmdModifyWorld_released",
                           G_CALLBACK(on_cmdModifyWorld_released));
  glade_xml_signal_connect(xmlWorldsListing, "on_cmdDuplicateWorld_released",
                           G_CALLBACK(on_cmdDuplicateWorld_released));
  glade_xml_signal_connect(xmlWorldsListing, "on_cmdConnectWorld_released",
                           G_CALLBACK(on_cmdConnectWorld_released));

  glade_xml_signal_connect(xmlWorldsListing, "on_tvWorlds_key_press",
                           G_CALLBACK(on_tvWorlds_key_press));

  return if_worlds_listing_get_widget("dlgWorldsListing");
}

world *
if_worlds_listing_new_world() {
  // Create a new empty world
  world *newWld = world_create(NULL);
  gint result = create_world_properties(newWld);

  if (result == GTK_RESPONSE_OK) {
    // Add the world, just add it well... also update the menu
    world_add(newWld);
    return newWld;
  } else {
    world_destroy(newWld);
    return NULL;
  }
}

world *
if_worlds_listing_modify_world(world * wld) {
  gint result;

  debug_msg(1, "if_worlds_listing_modify_world: modifying %s", options_get_str(
            wld->settings, "name"));

  result = create_world_properties(wld);

  if (result == GTK_RESPONSE_OK) {
    world_update(wld);
    return wld;
  } else {
    return NULL;
  }
}

// Switch the toggle button thingie
void
on_Autoload_toggled(GtkCellRendererToggle * cell, gchar * path_string,
                    gpointer user_data) {
  GtkTreeIter iter;
  GtkTreeModel *tvwModel;
  gchar *name;

  gboolean isActive = gtk_cell_renderer_toggle_get_active(cell);

  tvwModel =
    gtk_tree_view_get_model(GTK_TREE_VIEW
                            (if_worlds_listing_get_widget("tvWorlds")));

  gtk_tree_model_get_iter_from_string(tvwModel, &iter, path_string);
  gtk_tree_model_get(tvwModel, &iter, NAME_COLUMN, &name, -1);

  world *wld = world_get_by_name(name);

  g_free(name);
  options_set_int(wld->settings, "autoload", !isActive);
  options_save(wld->settings);

  gtk_list_store_set(GTK_LIST_STORE(tvwModel), &iter, AUTOLOAD_COLUMN,
                     !isActive, -1);
}

void
tvWorlds_delete_selected() {
  GtkTreeView *tree = GTK_TREE_VIEW(if_worlds_listing_get_widget("tvWorlds"));
  GtkTreeModel *model = gtk_tree_view_get_model(tree);
  GtkTreeSelection *selection = gtk_tree_view_get_selection(tree);
  GtkTreeIter iter;
  GList *paths = gtk_tree_selection_get_selected_rows(selection, &model);
  GList *refs = NULL;
  GList *node = NULL;
  gchar *name, *message;
  gpointer proxy = g_object_newv(G_TYPE_OBJECT, 0, NULL);
  world *wld;

  for (node = paths; node; node = node->next) {
    refs =
      g_list_append(refs,
                    gtk_tree_row_reference_new_proxy(proxy, model,
                                                     node->data));
    gtk_tree_path_free(node->data); /* Free unneeded tree path */
  }

  g_list_free(paths);           /* Emptied list */

  if (refs) {
    for (node = refs; node; node = node->next) {
      GtkTreePath *path = gtk_tree_row_reference_get_path(node->data);

      gtk_tree_model_get_iter(model, &iter, path);
      gtk_tree_model_get(model, &iter, NAME_COLUMN, &name, -1);

      // The name is now, well... okay. Now see if it is loaded
      wld = world_get_by_name(name);
      if (!(wld && wld->loaded)) {
        gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
        world_remove(name);
      } else {
        message =
          g_strdup_printf(_
                          ("Can't remove the world %s because it is loaded. First close the world and then try again!"),
                          name);
        gnoemoe_error_dialog(message,
                             GTK_WINDOW(if_worlds_listing_get_widget
                                        ("dlgWorldsListing")));
        g_free(message);
      }

      gtk_tree_row_reference_deleted(proxy, path);
      gtk_tree_path_free(path);
      gtk_tree_row_reference_free(node->data);  /* No longer needed */

      g_free(name);
    }
  } else {
    gnoemoe_error_dialog(_("You first need to select a world to delete!"),
                         GTK_WINDOW(if_worlds_listing_get_widget
                                    ("dlgWorldsListing")));
  }

  g_list_free(refs);
  g_object_unref(proxy);
}

// cmdDelete button released
void
on_cmdDeleteWorld_released(GtkButton * button, gpointer user_data) {
  tvWorlds_delete_selected();
}

void
on_tvWorlds_key_press(GtkWidget * widget, GdkEventKey * event,
                      gpointer user_data) {
  switch (event->keyval) {
  case GDK_D:
  case GDK_d:
    tvWorlds_delete_selected();
    break;
  case GDK_O:
  case GDK_o:
    break;
  case GDK_N:
  case GDK_n:
    on_cmdNewWorld_released(NULL, NULL);  // Its a bit dirty but what the hell
    break;
  case GDK_M:
  case GDK_m:
    on_cmdModifyWorld_released(NULL, NULL); // Its a bit dirty but what the hell
    break;
  }
}

void
on_cmdNewWorld_released(GtkButton * button, gpointer user_data) {
  world *newWld;
  GtkTreeIter iter;
  GtkListStore *store;

  newWld = if_worlds_listing_new_world();

  if (newWld) {
    // Now add a row to the tree
    store =
      GTK_LIST_STORE(gtk_tree_view_get_model
                     (GTK_TREE_VIEW
                      (if_worlds_listing_get_widget("tvWorlds"))));

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, AUTOLOAD_COLUMN,
                       options_get_int(newWld->settings, "autoload"),
                       NAME_COLUMN, options_get_str(newWld->settings, "name"),
                       -1);
  }
}

void
on_cmdModifyWorld_released(GtkButton * button, gpointer user_data) {
  // First try to find the first selected row
  GtkTreeView *tree = GTK_TREE_VIEW(if_worlds_listing_get_widget("tvWorlds"));
  GtkTreeModel *model = gtk_tree_view_get_model(tree);
  GtkTreeSelection *selection = gtk_tree_view_get_selection(tree);
  GtkTreeIter iter;
  GList *firstPath = gtk_tree_selection_get_selected_rows(selection, &model);
  gchar *name;

  if (firstPath) {
    gtk_tree_model_get_iter(model, &iter, firstPath->data);
    gtk_tree_model_get(model, &iter, NAME_COLUMN, &name, -1);
    world *wld = world_get_by_name(name);

    g_free(name);

    wld = if_worlds_listing_modify_world(wld);

    if (wld) {
      // Still need to update the treeview thingie
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, AUTOLOAD_COLUMN,
                         options_get_int(wld->settings, "autoload"),
                         NAME_COLUMN, options_get_str(wld->settings, "name"),
                         -1);
    }
  } else {
    gnoemoe_error_dialog(_("You first need to select a world to modify!"),
                         GTK_WINDOW(if_worlds_listing_get_widget
                                    ("dlgWorldsListing")));
  }

  g_list_free(firstPath);
}

void
on_cmdDuplicateWorld_released(GtkButton * button, gpointer user_data) {
  GtkTreeView *tree = GTK_TREE_VIEW(if_worlds_listing_get_widget("tvWorlds"));
  GtkTreeModel *model = gtk_tree_view_get_model(tree);
  GtkTreeSelection *selection = gtk_tree_view_get_selection(tree);
  GtkTreeIter iter;
  GList *firstPath = gtk_tree_selection_get_selected_rows(selection, &model);
  gchar *name;
  gint result;

  if (firstPath) {
    gtk_tree_model_get_iter(model, &iter, firstPath->data);
    gtk_tree_model_get(model, &iter, NAME_COLUMN, &name, -1);
    world *wld = world_get_by_name(name);

    // now duplicate the world
    world *newWld = world_copy(wld);

    // Make sure that we set the path to NULL, because otherwise it would look like
    // a modifying of the source world, we don't want that! Setting it to NULL will
    // cause it to act like a new world, but it has the same data as the source, neat!
    g_free(newWld->path);
    newWld->path = NULL;
    // Yeah we got some world, now show the props!
    result = create_world_properties(newWld);

    if (result == GTK_RESPONSE_OK) {
      // Add the world, just add it well... also update the menu
      world_add(newWld);

      // Now add a row to the tree
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, AUTOLOAD_COLUMN,
                         options_get_int(newWld->settings, "autoload"),
                         NAME_COLUMN, options_get_str(newWld->settings,
                                                      "name"), -1);

    }

  } else {
    gnoemoe_error_dialog(_("You first need to select a world to duplicate!"),
                         GTK_WINDOW(if_worlds_listing_get_widget
                                    ("dlgWorldsListing")));
  }

  g_list_free(firstPath);
}

void
on_cmdConnectWorld_released(GtkButton * button, gpointer user_data) {
  GtkTreeView *tree = GTK_TREE_VIEW(if_worlds_listing_get_widget("tvWorlds"));
  GtkTreeModel *model = gtk_tree_view_get_model(tree);
  GtkTreeSelection *selection = gtk_tree_view_get_selection(tree);
  GtkTreeIter iter;
  GList *firstPath = gtk_tree_selection_get_selected_rows(selection, &model);
  GList *path;
  gchar *name;

  if (firstPath) {
    for (path = firstPath; path; path = path->next) {
      gtk_tree_model_get_iter(model, &iter, firstPath->data);
      gtk_tree_model_get(model, &iter, NAME_COLUMN, &name, -1);
      world_load(world_get_by_name(name));
    }
  } else {
    gnoemoe_error_dialog(_("You first need to select a world to connect to!"),
                         GTK_WINDOW(if_worlds_listing_get_widget
                                    ("dlgWorldsListing")));
  }

  g_list_free(firstPath);
}
