#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <libgnomevfs/gnome-vfs.h>

#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>

#include "if_main.h"
#include "if_world.h"
#include "if_mcpconsole.h"

#ifdef HASRUBY
#  include "if_scripts.h"
#endif

#include "debug.h"
#include "my-string.h"
#include "gnoe_pixbuf.h"
#include "main.h"
#include "support.h"
#include "mcp/mcp.h"
#include "world.h"
#include "net.h"
#include "editor.h"
#include "script.h"

static char *appPath;
static char *worldsPath;
static char *settingsPath;

options *mainSettings;

static int dbgLevel = 0;
static gboolean showVersion = FALSE;
static gboolean recover = FALSE;
static gchar *loadWorlds = NULL;
static GnomeClient *client;

struct poptOption poptions[] = {
  {
   "debug",
   'd',
   POPT_ARG_INT,
   &dbgLevel,
   0,
   N_
   ("Enable debugging. Debugging can be done in different depth levels, each level inherits all messages from the levels below: 0 = None, 1 = Default and 2 = MCP"),
   N_("DBGGLEVEL")
   },
  {
   "version",
   'v',
   POPT_ARG_NONE,
   &showVersion,
   0,
   N_("Show application version"),
   NULL},
  {
   "load",
   'l',
   POPT_ARG_STRING,
   &loadWorlds,
   0,
   N_("Load specified worlds, seperated by a `,'"),
   N_("WORLDS")},
  {
   "recover",
   'r',
   POPT_ARG_NONE,
   &recover,
   0,
   N_("Recover from previous session (used with gnome session)"),
   NULL},
  {
   NULL,
   '\0',
   0,
   NULL,
   0,
   NULL,
   NULL}
};

static void
session_die(GnomeClient * client, gpointer client_data) {
  gtk_main_quit();
}

static gboolean
save_session(GnomeClient * client, gint phase, GnomeSaveStyle save_style,
             gint is_shutdown, GnomeInteractStyle interact_style,
             gint is_fast, gchar ** client_data) {
  gchar **argv;
  guint argc = sizeof(client_data) / sizeof(gchar *) + 1;
  int i;
  GString *ws = g_string_new("");
  GList *elem;
  world *wld;

  argv = g_new(gchar *, argc);
  argv[0] = client_data[0];
  argv[1] = "--recover";

  for (i = 1; i < argc - 1; i++) {
    argv[i + 1] = client_data[i];
  }

  gnome_client_set_clone_command(client, argc, argv);
  gnome_client_set_restart_command(client, argc, argv);

  // Saving worlds state
  for (elem = world_get_worlds(); elem; elem = elem->next) {
    wld = (world *) (elem->data);

    if (wld->loaded) {
      if (strlen(ws->str) != 0) {
        ws = g_string_append_c(ws, ';');
      }

      ws = g_string_append(ws, options_get_str(wld->settings, "name"));
    }
  }

  options_set_str(mainSettings, "worlds_saved_state", ws->str);
  options_save(mainSettings);
  g_string_free(ws, TRUE);

  return TRUE;
}

void
main_create_default_settings() {
  mainSettings = options_create();

  options_add(mainSettings, "editor_alternative", "0");
  options_add(mainSettings, "editor_embed", "0");
  options_add(mainSettings, "editor_needs_terminal", "0");


  options_add(mainSettings, "font-family", "Monospace 8");
  options_add(mainSettings, "bold-colors", "0");

  options_add(mainSettings, "fg_default", "#D6B5D6B5D6B5");
  options_add(mainSettings, "fg_black", "#2D6B2D6B2D6B");
  options_add(mainSettings, "fg_red", "#FFFF00000000");
  options_add(mainSettings, "fg_green", "#0000FFFF0000");
  options_add(mainSettings, "fg_yellow", "#FFFFD0450000");
  options_add(mainSettings, "fg_blue", "#3EF73EF7BFFF");
  options_add(mainSettings, "fg_purple", "#A0A02020F0F0");
  options_add(mainSettings, "fg_cyan", "#0000FFFFFFFF");
  options_add(mainSettings, "fg_white", "#D8C5D8C5D8C5");

  options_add(mainSettings, "fg_default_h", "#FFFFFFFFFFFF");
  options_add(mainSettings, "fg_black_h", "#529452945294");
  options_add(mainSettings, "fg_red_h", "#FFFF785F785F");
  options_add(mainSettings, "fg_green_h", "#66ADFFFF66AD");
  options_add(mainSettings, "fg_yellow_h", "#FFFFFFFF58C6");
  options_add(mainSettings, "fg_blue_h", "#86318631FFFF");
  options_add(mainSettings, "fg_purple_h", "#C6576A18FFFF");
  options_add(mainSettings, "fg_cyan_h", "#86EEFFFFFFFF");
  options_add(mainSettings, "fg_white_h", "#FFFFFFFFFFFF");

  options_add(mainSettings, "bg_default", "#000000000000");
  options_add(mainSettings, "bg_black", "#2B5B2B5B2B5B");
  options_add(mainSettings, "bg_red", "#FFFF00000000");
  options_add(mainSettings, "bg_green", "#000080000000");
  options_add(mainSettings, "bg_yellow", "#C047C0470000");
  options_add(mainSettings, "bg_blue", "#00000000FFFF");
  options_add(mainSettings, "bg_purple", "#A0A02020F0F0");
  options_add(mainSettings, "bg_cyan", "#0000B74CB74C");
  options_add(mainSettings, "bg_white", "#FFFFFFFFFFFF");

  options_add(mainSettings, "background_transparent", "0");
  options_add(mainSettings, "background_transparancy", "100");

  // Non configurable options
  options_add(mainSettings, "worlds_saved_state", "");

}

void
main_load_settings() {
  main_create_default_settings();

  options_load(mainSettings, settingsPath);
}

options *
main_get_options() {
  return mainSettings;
}

void
main_directory_remove_all(gchar * path, gboolean removeSelf) {
  GDir *cDir;
  gchar *name;
  gchar *newPath;

  if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
    // Iterate through the files and do the right thingie
    if ((cDir = g_dir_open(path, 0, NULL))) {
      while ((name = (gchar *) (g_dir_read_name(cDir))) != NULL) {
        newPath = g_strconcat(path, "/", name, NULL);
        main_directory_remove_all(newPath, TRUE);
        g_free(newPath);
      }
      g_dir_close(cDir);
    }
  }

  if (removeSelf) {
    // Its a file, or just empty! MUST...BE...REMOVEEEED!
    remove(path);
  }
}

gchar *
main_get_worlds_path() {
  return worldsPath;
}

gchar *
main_get_app_path() {
  return appPath;
}

void
make_directorys() {
  mkdir(appPath, 0755);
  mkdir(worldsPath, 0755);
}

void
main_fini() {
  options_save(mainSettings);

  gnome_vfs_shutdown();
  mcp_fini();
  editor_fini();
  
  #ifdef HASRUBY
    script_fini();
    if_scripts_fini();
  #endif
  
  mcpconsole_fini();
  world_destroy_worlds();
  gnoe_pixbuf_fini();
  
  options_save(mainSettings);
  options_destroy(mainSettings);
  g_free(appPath);
  g_free(worldsPath);
  g_free(settingsPath);
}

void
main_init() {
  GtkWidget *wndMain;
  gchar **wrlds, *savedState;
  int i = 0, width, height;
  world *wld;

  debug_set_level(dbgLevel);
  gnoe_pixbuf_init();

  worldsPath = g_strconcat(appPath, "/worlds", NULL);
  settingsPath = g_strconcat(appPath, "/settings", NULL);

  if (!g_file_test(appPath, G_FILE_TEST_EXISTS)) {
    make_directorys();
  }
  
  // Create the main window
  wndMain = create_wndMain();
  main_load_settings();
  
  width = options_get_int(mainSettings, "width");
  height = options_get_int(mainSettings, "height");
  
  if (height > 10 && width > 10) {
    gtk_window_set_default_size(GTK_WINDOW(wndMain), width, height);
  }
  
  gtk_widget_show(wndMain);

  // Gray out some menu items that can't be used without an active world
  interface_no_worlds_loaded();
  mcpconsole_init();

  #ifdef HASRUBY
    if_scripts_init();
    script_init();
  #endif

  world_load_worlds(worldsPath, !(recover || loadWorlds));
  
  if (recover) {
    // We need to recover, open all worlds in "worlds_saved_state"
    savedState = options_get_str(mainSettings, "worlds_saved_state");

    if (strlen(savedState) != 0) {
      wrlds = g_strsplit(savedState, ";", -1);

      for (i = 0; wrlds[i]; i++) {
        if (strlen(wrlds[i]) != 0) {
          wld = world_get_by_name(wrlds[i]);

          if (wld) {
            world_load(wld);
          }
        }
      }

      g_strfreev(wrlds);
    }
  } else if (loadWorlds) {
    wrlds = g_strsplit(loadWorlds, ",", -1);
    
    for (i = 0; wrlds[i]; i++) {
      wld = world_get_by_name(wrlds[i]);
      
      if (wld) {
        world_load(wld);
      }
    }
    
    g_strfreev(wrlds);
  }
}

int
main(int argc, char *argv[]) {
  if (!g_file_test(G_XML, G_FILE_TEST_EXISTS)) {
    printf(_
           ("main: the interface glade xml file could not be found (%s), make sure you 'make install' first!\n"),
           G_XML);
    return 0;
  }

  if (!(appPath = gnome_util_home_file("gnoemoe"))) {
    printf(_("main: there is no app dir, must... QUIT!\n"));
    return 0;
  }
#ifdef ENABLE_NLS
  bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
  textdomain(GETTEXT_PACKAGE);
#endif

  gtk_set_locale();

  gtk_init(&argc, &argv);

  gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                     argc, argv, GNOME_PARAM_POPT_TABLE, poptions,
                     GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);

  if (showVersion) {
    printf(_("Current version of GnoeMoe is %s\n"), VERSION);
    return 0;
  }

  client = gnome_master_client();

  gtk_signal_connect(GTK_OBJECT(client), "save_yourself",
                     GTK_SIGNAL_FUNC(save_session), argv);
  gtk_signal_connect(GTK_OBJECT(client), "die", GTK_SIGNAL_FUNC(session_die),
                     NULL);

  gnome_vfs_init();
  glade_init();
  mcp_init();
  main_init();
  editor_init();
                    
  gtk_main();

  main_fini();
  return 0;
}
