#ifndef MCP_H
#define MCP_H 1

#include <gtk/gtk.h>
#include "world.h"
#include "support.h"

typedef struct _mcp_session mcp_session;
typedef struct _mcp_package mcp_package;
typedef struct _package_registration package_registration;

typedef void (*mcp_init_function) (mcp_package *);
typedef void (*mcp_fini_function) (mcp_package *);
typedef void (*mcp_handle_simple_function) (mcp_package *, gchar *, GList *);

/* gboolean mcp_handle_multi_function(mcp_package *caller, gchar *data_key, gchar *key, gchar *value, GList *allValues
   Can be used to handle single multiline values (by returning TRUE and inspecting key/value
   Can be used to handle the total of multiline values when multiline ends (by returning FALSE as long as allValues == NULL
*/
typedef gboolean(*mcp_handle_multi_function) (mcp_package *, gchar *, gchar *,
                                              gchar *, GList *);

// mcp_session is used to hold packages supported by a world
struct _mcp_session {
  world *owner;
  gchar *authkey;
  double version;

  GList *packages;
};

// package_registration is used to hold information on package handler functions, version, name and aliases
struct _package_registration {
  gchar *name;
  double minVersion;
  double maxVersion;

  mcp_init_function initFunc;
  mcp_fini_function finiFunc;
  mcp_handle_simple_function simpleFunc;
  mcp_handle_multi_function multiFunc;
  
  GList *depends;
  GList *overrides;
};

typedef struct _mcp_multiline_info {
  gchar *key;
  gchar *data_tag;
  GList *data;
} mcp_multiline_info;

// mcp_package struct holds information of its ancestor general package (registration)
// it also holds information on the session it is packed into
// Also, it contains a gpointer user_data field which contains session/package specific information
struct _mcp_package {
  package_registration *registration;
  mcp_session *session;

  GList *multiline;

  double version;
  gpointer user_data;
};

typedef struct _message_info {
  gchar *name;
  gchar *authkey;
  GList *fields;
} message_info;


void mcp_handle_out_of_bound(world * wld, gchar * line);
void mcp_register_package(gchar * name, double minVersion, double maxVersion,
                          mcp_init_function, mcp_handle_simple_function,
                          mcp_handle_multi_function, mcp_fini_function, 
                          gchar *overrides, gchar *depends);
void mcp_init();
void mcp_fini();

void mcp_send_simple(mcp_session * session, gchar * pname, gchar * firstKey,
                     ...);
void mcp_send_multiline(mcp_session * session, gchar * data_tag, gchar * key,
                        GList * lines);

void mcp_destroy_fields(GList * fields);
gchar *mcp_find_value(GList * fields, gchar * key);
package_registration *mcp_find_package(gchar * pname);
mcp_package *mcp_create_package_for(mcp_session * session,
                                    package_registration * p);

GList *mcp_get_registry();
double mcp_get_version(double clientMin, double clientMax, double serverMin,
                       double serverMax);

void mcp_debug_print_fields(GList * fields);
mcp_session *mcp_find_session(world * wld);
mcp_package *mcp_session_find_package(mcp_session *session, gchar *name);

gchar *mcp_generate_data_tag();
gchar *mcp_un_escape(gchar * line);
void mcp_close(world * wld);

#endif
