#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include "mcp.h"
#include "debug.h"
#include "packages.h"
#include "if_mcpconsole.h"

typedef struct _vmoo_client_info {
  int width;
  int height;
  int wmargin;
  int hmargin;
  
  gulong handler;
} vmoo_client_info;

void
mcp_vmoo_client_resized(GtkWidget * widget, GtkAllocation * allocation,
                        mcp_package * p) {
  vmoo_client_info *i = (vmoo_client_info *) (p->user_data);
  int width, height;
  gchar w[10], h[10];
  world *own = p->session->owner;
             
  width = (allocation->width - i->wmargin);
  height = (allocation->height - i->hmargin);

  if ((width % own->characterWidth) != 0) {
    width = (width / own->characterWidth);
  } else {
    width = (width / own->characterWidth) - 1;
  }

  if ((height % own->characterHeight) != 0) {
    height = (height / own->characterHeight);
  } else {
    height = (height / own->characterHeight) - 1;
  }
  
  if (i->width != width || i->height != height) {
    snprintf(w, 9, "%d", width);
    snprintf(h, 9, "%d", height);
    mcp_send_simple(p->session, "dns-com-vmoo-client-screensize", "cols", w,
                    "rows", h, NULL);
    i->width = width;
    i->height = height;
  }
}


void
mcp_vmoo_client_fini(mcp_package * p) {
  vmoo_client_info *i = (vmoo_client_info *) (p->user_data);

  if (p->session->owner->widgets.txtVwWorld) {
    g_signal_handler_disconnect((gpointer)
                                (p->session->owner->widgets.txtVwWorld),
                                i->handler);
  }

  g_free(i);
}

void
mcp_vmoo_client_init(mcp_package * p) {
  // Connect a handler to the size-allocate of the TextView!
  vmoo_client_info *i = g_new(vmoo_client_info, 1);
  world *own = p->session->owner;
  
  i->width = 0;
  i->height = 0;

  i->wmargin = gtk_text_view_get_left_margin(
             GTK_TEXT_VIEW(own->widgets.txtVwWorld)) +
           gtk_text_view_get_right_margin(
             GTK_TEXT_VIEW(own->widgets.txtVwWorld));

  i->hmargin = gtk_text_view_get_left_margin(
             GTK_TEXT_VIEW(own->widgets.txtVwWorld)) +
           gtk_text_view_get_right_margin(
             GTK_TEXT_VIEW(own->widgets.txtVwWorld));


  i->handler =
    g_signal_connect((gpointer) (own->widgets.txtVwWorld),
                     "size-allocate", G_CALLBACK(mcp_vmoo_client_resized), p);

  p->user_data = (gpointer) i;
  mcp_send_simple(p->session, "dns-com-vmoo-client-info", "name", "GnoeMoe",
                  "text-version", IVERSION, "internal-version", "0001",
                  "reg-id", "0", "flags", "", NULL);
}

void
mcp_vmoo_client_simple(mcp_package * p, gchar * suffix, GList * fields) {

}
