#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include "mcp.h"
#include "debug.h"
#include "if_mcpconsole.h"
#include "world.h"
#include "editor.h"

typedef struct _simpleedit_info {
  gchar *ref;
  gchar *name;
  gchar *type;
} simpleedit_info;

void
mcp_simpleedit_init(mcp_package * p) {
  simpleedit_info *i = g_new(simpleedit_info, 1);

  i->ref = NULL;
  i->name = NULL;
  i->type = NULL;

  p->user_data = (gpointer) i;
}

void
mcp_simpleedit_fini(mcp_package * p) {
  simpleedit_info *i = (simpleedit_info *) (p->user_data);

  g_free(i->ref);
  g_free(i->name);
  g_free(i->type);
  g_free(i);
}

void
mcp_simpleedit_simple(mcp_package * p, gchar * suffix, GList * fields) {
  gchar *name, *ref, *type;
  simpleedit_info *i = (simpleedit_info *) (p->user_data);

  if (g_strcasecmp(suffix, "content") == 0) {
    if (!(name = mcp_find_value(fields, "name"))) {
      debug_msg(2, "mcp_simpleedit_simple: no name found, ignoring!");
    } else if (!(ref = mcp_find_value(fields, "reference"))) {
      debug_msg(2, "mcp_simpleedit_simple: no reference found, ignoring!");
    } else if (!(type = mcp_find_value(fields, "type"))) {
      debug_msg(2, "mcp_simpleedit_simple: no type found, ignoring!");
    } else {
      debug_msg(2,
                "mcp_simpleedit_simple: accept content: name = %s, reference = %s, type = %s",
                name, ref, type);
      i->name = g_strdup(name);
      i->ref = g_strdup(ref);
      i->type = g_strdup(type);
    }
  } else {
    debug_msg(2, "mcp_simpleedit_simple: unknown suffix: %s", suffix);
  }
}

gboolean
mcp_simpleedit_multi(mcp_package * p, gchar * data_tag, gchar * key,
                     gchar * value, GList * allValues) {
  simpleedit_info *i = (simpleedit_info *) (p->user_data);

  if (key) {
    return FALSE;
  } else {
    // All values arrived! Yeah!
    editor_invoke_mcp(p->session->owner, i->name, i->ref, i->type, allValues);
  }

  return TRUE;
}

void
mcp_simpleedit_save(world * wld, gchar * reference, gchar * type,
                    GList * lines) {
  mcp_session *session = mcp_find_session(wld);
  gchar *data_tag = mcp_generate_data_tag();

  if (session) {
    mcp_send_simple(session, "dns-org-mud-moo-simpleedit-set", "reference",
                    reference, "type", type, "content*", "", "_data-tag",
                    data_tag, NULL);
    mcp_send_multiline(session, data_tag, "content", lines);
  }

  g_free(data_tag);
}
