#include <gtk/gtk.h>
#include <glib.h>
#include <string.h>

#include "debug.h"
#include "mcp.h"
#include "packages.h"
#include "list.h"
#include "gnoe_pixbuf.h"
#include "my-string.h"
#include "support.h"

#define DEFAULT_ICON_MAP "default.map"

static const keyvaluepair icon_mapping[] = {
  {"newbie", "userlist/newbie.svg"},
  {"inhabitant", "userlist/inhabitant.svg"},
  {"inhabitant+", "userlist/inhabitantplus.svg"},
  {"schooled", "userlist/schooled.svg"},
  {"key", "userlist/key.svg"},
  {"star", "userlist/star.svg"},
  {"wizard", "userlist/wizard.svg"}
};

typedef enum _user_state {
  U_NORMAL,
  U_AWAY,
  U_IDLE,
  U_IDLEAWAY
} user_state;

typedef struct _userinfo {
  int nr;
  int icon;
  gchar *name;
  user_state state;
} userinfo;

typedef struct _userlist_info {
  int you;
  GList *fields;
  GList *icons;
  GList *users;
} userlist_info;

void
mcp_vmoo_userlist_init(mcp_package * p) {
  userlist_info *i = g_new(userlist_info, 1);

  i->you = 0;
  i->fields = NULL;
  i->icons = NULL;
  i->users = NULL;

  world_userlist_create(p->session->owner);

  p->user_data = (gpointer) i;
}

void
userlist_remove_users(userlist_info * i) {
  GList *users = g_list_copy(i->users);
  GList *field;

  for (field = users; field; field = field->next) {
    i->users = g_list_remove(i->users, field->data);
    g_free(((userinfo *) (field->data))->name);
    g_free((userinfo *) (field->data));
  }

  g_list_free(users);
}

void
mcp_vmoo_userlist_fini(mcp_package * p) {
  userlist_info *i = (userlist_info *) (p->user_data);
  GList *field;

  for (field = i->fields; field; field = field->next) {
    g_free((gchar *) (field->data));
  }

  g_list_free(i->fields);

  for (field = i->icons; field; field = field->next) {
    g_free((gchar *) (field->data));
  }

  g_list_free(i->icons);

  userlist_remove_users(i);
  g_list_free(i->users);

  world_userlist_free(p->session->owner);

  g_free(i);
}

void
mcp_vmoo_userlist_simple(mcp_package * p, gchar * suffix, GList * fields) {
  MOOVar *you;
  userlist_info *i = (userlist_info *) (p->user_data);

  if (suffix && strcmp(suffix, "you") == 0) {
    you = MOOVar_parse(mcp_find_value(fields, "nr"));
    if (you->type == OBJECT) {
      i->you = you->i;
      MOOVar_free(you);
    } else {
      debug_msg(2, "mcp_vmoo_userlist_you: you is not an object!");
    }

  }
}

void
handle_fields(mcp_package * p, MOOVar * list) {
  MOOVar *field;
  userlist_info *i = (userlist_info *) (p->user_data);

  for (field = list->list; field; field = field->next) {
    if (field->type == STRING) {
      i->fields = g_list_append(i->fields, g_strdup(field->s));
    }
  }
}

GdkPixbuf *
get_icon(userlist_info * i, userinfo * ui) {
  gchar *field;
  int p;
  int index = ui->icon;

  if (index < 1 || index > g_list_length(i->icons)) {
    return NULL;
  } else {
    switch (ui->state) {
    case U_NORMAL:
      field = (gchar *) (g_list_nth_data(i->icons, index - 1));

      for (p = 0; p < sizeof(icon_mapping) / sizeof(keyvaluepair); p++) {
        if (strcasecmp(icon_mapping[p].key, field) == 0) {
          return gnoe_pixbuf_get(icon_mapping[p].value);
        }
      }
      
      break;
    case U_IDLE:
      return gnoe_pixbuf_get("userlist/idle.svg");
      break;
    case U_AWAY:
      return gnoe_pixbuf_get("userlist/away.svg");
      break;
    case U_IDLEAWAY:
      return gnoe_pixbuf_get("userlist/idleaway.svg");
      break;
    default:
      break;
    }
  }

  return NULL;
}

void
handle_icons(mcp_package * p, MOOVar * list) {
  MOOVar *field;
  userlist_info *i = (userlist_info *) (p->user_data);

  for (field = list->list; field; field = field->next) {
    if (field->type == STRING) {
      i->icons = g_list_append(i->icons, g_strdup(field->s));
    }
  }
}

gchar *
userlist_get_string(MOOVar * v, GList * fields, MOOType type, gchar * cmp) {
  while (fields && strcasecmp(((gchar *) (fields->data)), cmp) != 0) {
    fields = fields->next;

    if (v) {
      v = v->next;
    }
  }

  if (v && v->type == type) {
    return g_strdup(v->s);
  }

  return NULL;
}

int
userlist_get_int(MOOVar * v, GList * fields, MOOType type, gchar * cmp) {
  while (fields && strcasecmp(((gchar *) (fields->data)), cmp) != 0) {
    fields = fields->next;

    if (v) {
      v = v->next;
    }
  }

  if (v && v->type == type) {
    return v->i;
  }

  return 0;
}

userinfo *
find_user(GList * users, int nr) {
  userinfo *result;

  for (; users; users = users->next) {
    result = (userinfo *) (users->data);
    if (result->nr == nr) {
      return result;
    }
  }

  return NULL;

}

void
userlist_remove_user(mcp_package *p, int nr) {
  userlist_info *i = (userlist_info *)(p->user_data);
  GList *elem;
  userinfo *ui;

  for (elem = i->users; elem; elem = elem->next) {
    ui = (userinfo *) (elem->data);
    if (ui->nr == nr) {
      i->users = g_list_remove(i->users, ui);
      break;
    }
  }

  world_userlist_remove_user(p->session->owner, nr);
}

void
userlist_update_user(mcp_package *p, userinfo * ui) {
  userlist_info *i = (userlist_info *)(p->user_data);
  world_uu uuinfo;
  unsigned int sortid = 0;

  uuinfo.nr = ui->nr;
  uuinfo.name = g_strdup_printf("%s (#%d)", ui->name, ui->nr);

  switch (ui->state) {
    case U_NORMAL:
      sortid = g_list_length(i->icons) - ui->icon;
      break;
    case U_AWAY:
      sortid = g_list_length(i->icons) + 1;
      break;
    case U_IDLE:
      sortid = g_list_length(i->icons) + 2;
      break;
    case U_IDLEAWAY:
      sortid = g_list_length(i->icons) + 3;
      break;
  }
  
  uuinfo.sort =
    g_strdup_printf("%.2d%s", sortid,
                    uuinfo.name);
  uuinfo.icon = get_icon(i, ui);
  
  world_userlist_update_user(p->session->owner, &uuinfo);

  g_free(uuinfo.name);
  g_free(uuinfo.sort);
}

void
handle_user_update(mcp_package * p, MOOVar * v) {
  userlist_info *i = (userlist_info *) (p->user_data);
  int nr = userlist_get_int(v, i->fields, OBJECT, "object");
  gchar *name = userlist_get_string(v, i->fields, STRING, "name");
  int icon = userlist_get_int(v, i->fields, INT, "icon");
  userinfo *ui;

  if ((ui = find_user(i->users, nr)) == NULL) {
    // Add the user
    ui = g_new(userinfo, 1);
    ui->state = U_NORMAL;
    i->users = g_list_append(i->users, ui);
  }

  debug_msg(2, "handle_user_update: user updated: %s", name);

  ui->name = name;
  ui->nr = nr;
  ui->icon = icon;

  userlist_update_user(p, ui);
}

void
handle_set(mcp_package * p, MOOVar * v) {
  MOOVar *ui;
  userlist_info *i = (userlist_info *) (p->user_data);

  if (v->type != LIST) {
    return;
  }

  userlist_remove_users(i);
  world_userlist_clear(p->session->owner);


  for (ui = v->list; ui; ui = ui->next) {
    if (ui->type != LIST) {
      return;
    }

    handle_user_update(p, ui->list);
  }
}

void
handle_add(mcp_package * p, MOOVar * v) {
  if (v->type != LIST) {
    return;
  }

  handle_user_update(p, v->list);
}

void
handle_remove(mcp_package * p, MOOVar * v) {
  MOOVar *nr;

  if (v->type != LIST) {
    return;
  }

  for (nr = v->list; nr; nr = nr->next) {
    if (nr->type == OBJECT) {
      userlist_remove_user(p, nr->i);
    }
  }
}

void
handle_update(mcp_package * p, MOOVar * v) {
  if (v->type != LIST) {
    return;
  }

  handle_user_update(p, v->list);
}

void
handle_state(mcp_package * p, MOOVar * v, user_state state, gboolean onOff) {
  MOOVar *nr;
  userlist_info *i = (userlist_info *) (p->user_data);
  userinfo *u;

  if (v->type != LIST) {
    return;
  }

  for (nr = v->list; nr; nr = nr->next) {
    if (nr->type == OBJECT) {
      if ((u = find_user(i->users, nr->i))) {
        switch (state) {
        case U_IDLE:
          if (u->state == U_IDLEAWAY && !onOff) {
            u->state = U_AWAY;
          } else if (u->state == U_AWAY && onOff) {
            u->state = U_IDLEAWAY;
          } else if (onOff) {
            u->state = U_IDLE;
          } else if (u->state == U_IDLE) {
            u->state = U_NORMAL;
          }
          break;
        case U_AWAY:
          if (u->state == U_IDLEAWAY && !onOff) {
            u->state = U_IDLE;
          } else if (u->state == U_IDLE && onOff) {
            u->state = U_IDLEAWAY;
          } else if (onOff) {
            u->state = U_AWAY;
          } else if (u->state == U_AWAY) {
            u->state = U_NORMAL;
          }
          break;
        default:
          break;
        }

        userlist_update_user(p, u);
      }
    }
  }
}

gboolean
mcp_vmoo_userlist_multi(mcp_package * p, gchar * data_tag, gchar * key,
                        gchar * value, GList * allValues) {
  MOOVar *v = NULL;

  if (key) {
    if (strcmp(key, "fields") == 0) {
      v = MOOVar_parse(value);
      if (!v || v->type != LIST) {
        debug_msg(2, "mcp_vmoo_userlist_fields: invalid value: %s", value);
      } else {
        handle_fields(p, v);
      }
    } else if (strcmp(key, "icons") == 0) {
      v = MOOVar_parse(value);
      if (!v || v->type != LIST) {
        debug_msg(2, "mcp_vmoo_userlist_icons: invalid value: %s", value);
      } else {
        handle_icons(p, v);
      }
    } else if (strcmp(key, "d") == 0) {
      v = MOOVar_parse(value + 1);
      if (!v) {
        debug_msg(2, "mcp_vmoo_userlist_d: invalid value: %s", value + 1);
      } else {
        switch (value[0]) {
        case '=':
          handle_set(p, v);
          break;
        case '+':
          handle_add(p, v);
          break;
        case '-':
          handle_remove(p, v);
          break;
        case '*':
          handle_update(p, v);
          break;
        case '<':
          handle_state(p, v, U_IDLE, TRUE);
          break;
        case '>':
          handle_state(p, v, U_IDLE, FALSE);
          break;
        case '[':
          handle_state(p, v, U_AWAY, TRUE);
          break;
        case ']':
          handle_state(p, v, U_AWAY, FALSE);
          break;
        case '(':
          handle_remove(p, v);
          break;
        default:
          break;
        }
      }
    }
  }

  if (v) {
    MOOVar_free(v);
  }
  return TRUE;
}
