#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <gtk/gtk.h>

#include "mcp.h"
#include "debug.h"
#include "packages.h"
#include "if_mcpconsole.h"

void mcp_awns_status_simple(mcp_package * p, char *suffix, GList * fields);
void mcp_mcpreset_simple(mcp_package * p, gchar * suffix, GList * fields);

void
packages_init() {
  mcp_register_package("mcp-negotiate", 1.0, 2.0, mcp_negotiate_init,
                       mcp_negotiate_simple, NULL, mcp_negotiate_fini, 
                       NULL, NULL);
  mcp_register_package("dns-com-awns-status", 1.0, 1.0, NULL,
                       mcp_awns_status_simple, NULL, NULL, NULL, NULL);
  mcp_register_package("dns-com-vmoo-client", 1.0, 1.0, mcp_vmoo_client_init,
                       mcp_vmoo_client_simple, NULL, mcp_vmoo_client_fini,
                       NULL, NULL);
  mcp_register_package("dns-org-mud-moo-simpleedit", 1.0, 1.0,
                       mcp_simpleedit_init, mcp_simpleedit_simple,
                       mcp_simpleedit_multi, mcp_simpleedit_fini,
                       NULL, NULL);
  mcp_register_package("dns-com-vmoo-userlist", 1.0, 1.0,
                       mcp_vmoo_userlist_init, mcp_vmoo_userlist_simple,
                       mcp_vmoo_userlist_multi, mcp_vmoo_userlist_fini,
                       NULL, NULL);
  mcp_register_package("dns-nl-icecrew-mcpreset", 1.0, 1.0, NULL,
                       mcp_mcpreset_simple, NULL, NULL, NULL, NULL);
  mcp_register_package("dns-com-awns-ping", 1.0, 1.0, NULL,
                       mcp_ping_simple, NULL, NULL, NULL, NULL);
  mcp_register_package("dns-nl-icecrew-playerdb", 1.0, 1.0, mcp_playerdb_init,
                       mcp_playerdb_simple, mcp_playerdb_multi, 
                       mcp_playerdb_fini, NULL, NULL);
  mcp_register_package("dns-nl-icecrew-userlist", 1.0, 1.0, mcp_ice_userlist_init,
                       NULL, mcp_ice_userlist_multi, mcp_ice_userlist_fini, 
                       "dns-com-vmoo-userlist", "dns-nl-icecrew-playerdb");
}

/* AWNS STATUS CALLBACKS */

void
mcp_awns_status_simple(mcp_package * p, gchar * suffix, GList * fields) {
  gchar *text = mcp_find_value(fields, "text");
  gchar *status =
    g_convert(text, strlen(text), "UTF-8",
              options_get_str(p->session->owner->settings, "charset"),
              NULL, NULL, NULL);

  if (status) {
    world_update_status(p->session->owner, status);
    g_free(status);
  } else {
    world_update_status(p->session->owner, text);
  }
}

/* MCPRESET PACKAGE CALLBACKS */

void
mcp_mcpreset_simple(mcp_package * p, gchar * suffix, GList * fields) {
  if (strcmp(suffix, "reset") == 0) {
    mcp_close(p->session->owner);
  }
}

gchar *
mcp_jump_space(gchar *s) {
  while (*s != '\0' && *s == ' ') {
    *s++;
  }
  
  return s;
}

GList *
mcp_parse_list(gchar *s) {
  GList *result = NULL;
  gchar item[strlen(s) + 1];
  int i, p;  
  
  
  while (s[0] != '\0') {
    s = mcp_jump_space(s);
    i = 0;
    p = 0;
    item[0] = '\0';

    while (s[i] != '\0' && s[i] != ' ') {
      if (s[i] == '\\' && s[i + 1] != '\0') {
        i++;
      }

      item[p] = s[i];
      p++;
      i++;
    }

    item[p] = '\0';
    result = g_list_append(result, g_strdup(item));
    s = s + i;
  }

  return result;
}

void 
mcp_free_list(GList *l) {
  GList *temp;
  
  for (temp = l; temp; temp = temp->next) {
    g_free(temp->data);
  }
  
  g_list_free(l);
}
