#ifndef SUPPORT_H
#define SUPPORT_H 1

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <libgnomevfs/gnome-vfs.h>

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#ifndef _
#  define _(String) (String)
#endif
#  define N_(String) (String)
#endif

#define CALC_COLOR_RANGE(x) (int)((-(1/((x * 0.5)+1))+1)*255)

/** \defgroup support */

/** \ingroup support 
 *  \brief Widget container for tabs
 *
 *  Contains all widgets that could be useful to be modified.
 *  Stored here because it makes it easier to look them up.
 *  
 */
 
typedef struct _label_info {
  /** \brief GtkButton widget
    *
    * GtkButton widget, reference to the tabs exit button
    */
  GtkWidget *tabExitButton;

    /** \brief GtkImage widget
    *
    * GtkImage widget, reference to the tabs exit button image
    */
  GtkWidget *tabExitImage;  
  
  /** \brief GtkLabel widget
    *
    * GtkLabel widget, reference to the tabs label
    */
  GtkWidget *tabLabel;
  
  /** \brief GtkImage widget
    *
    * GtkImage widget, reference to the tabs icon
    */
  GtkWidget *tabIcon;
} label_info;

/** \ingroup support
 *  \brief Key/value pair struct
 *
 *  Contains two fields. Used to create hash tables
 *  
 */
typedef struct _keyvaluepair {
  gchar *key; /**< the key */
  gchar *value; /**< the value */
} keyvaluepair;

typedef struct _GnoeMoeFetchHandle {
  GnomeVFSAsyncHandle *handle;
  
  GFunc cb;
  gpointer user_data;

	GList *source_uri;
	GList *dest_uri;
	  
  GnomeVFSXferPhase cur_phase;
  GnomeVFSXferPhase prev_phase;
  GnomeVFSXferProgressStatus prev_status;

	GnomeVFSFileSize bytes_total;
	GnomeVFSFileSize file_size;
	GnomeVFSFileSize bytes_copied;
	GnomeVFSFileSize total_bytes_copied;
	
	gulong files_total;  
  gulong cur_file;
  gulong prev_file;
  gchar *cur_file_name;
  
  gboolean aborted;
  gboolean done;
} GnoeMoeFetchHandle;

gchar *fix_decimal_point(gchar *line, int len);
gchar *fix_decimal_point_rev(gchar *line, int len);
gchar *ansi_strip(gchar * s);
int garray_length(gchar **s);
void g_list_free_simple(GList *s);
gchar *g_list_find_simple(GList *s, gchar *f);

void gnoemoe_error_dialog(gchar * message, GtkWindow * parent);
void gnoemoe_warning_dialog(gchar * message, GtkWindow * parent);
void gnoemoe_info_dialog(gchar * message, GtkWindow * parent);
void gnoemoe_questio_ndialog(gchar * message, GtkWindow * parent);

void doEvents();

gchar *str_escape(gchar * line);
gint gnoemoe_url_regex_match(const gchar *msg, int len, GArray *start,  GArray *end);
void gnoemoe_open_url (const gchar *url);

GnoeMoeFetchHandle * gnoemoe_fetch(const GList *source, const GList *dest, 
                                   GFunc cb, gpointer user_data);
void gnoemoe_fetch_handle_free(GnoeMoeFetchHandle *g);
#endif
